/*! \file EnuTypeQuelParticulier.h
    \brief def de l'enuméré concernant les types quelconques particuliers.
*/
// FICHIER : EnuTypeQuelParticulier.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des types de grandeurs sont
// stockes a l'aide d'un type enumere. Les fonctions NomTypeQuelParticulier et Id_nomTypeQuelParticulier rendent
// possible le lien entre les noms des geometries et les identificateurs
// de type enumere correspondants.


#ifndef ENUTYPEQUELPARTICULIER_H
#define ENUTYPEQUELPARTICULIER_H
#include <iostream>
using namespace std;
#include <map>
#include "ParaGlob.h"
#include "EnumTypeGrandeur.h"
#include "Enum_dure.h"
#include "Tableau_T.h"


/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types quelconques particuliers.

enum EnuTypeQuelParticulier { RIEN_TYPE_QUELCONQUE_PARTICULIER = 1,PARTICULIER_TENSEURHH
                         ,PARTICULIER_TENSEURBB,PARTICULIER_COORDONNEE
                         ,PARTICULIER_TABLEAU_COORDONNEE
                         ,PARTICULIER_VECTEUR,PARTICULIER_TABLEAU_VECTEUR
                         ,PARTICULIER_BASE_H,PARTICULIER_BASE_B
                         ,PARTICULIER_TABLEAU_BASE_H,PARTICULIER_TABLEAU_BASE_B
                         ,PARTICULIER_TABLEAU_TENSEURHH,PARTICULIER_TABLEAU2_TENSEURHH
                         ,PARTICULIER_TENSEURBH,PARTICULIER_SCALAIRE_DOUBLE
                         ,PARTICULIER_TABLEAU_SCALAIRE_DOUBLE
                         ,PARTICULIER_SCALAIRE_ENTIER,PARTICULIER_TABLEAU_SCALAIRE_ENTIER
                         ,PARTICULIER_TABLEAU_QUELCONQUE
								                 ,PARTICULIER_TABLEAU_TENSEURBH,PARTICULIER_TABLEAU_TENSEURBB
								                 ,PARTICULIER_TENSEURHB,PARTICULIER_TABLEAU_TENSEURHB
                         ,PARTICULIER_DDL_ETENDU,PARTICULIER_TABLEAU_DDL_ETENDU
                         ,PARTICULIER_VECTEUR_NOMMER,PARTICULIER_TABLEAU_VECTEUR_NOMMER
                         ,PARTICULIER_SCALAIRE_DOUBLE_NOMMER_INDICER
                            };
/// @}  // end of group


/// @addtogroup Group_types_enumeres
///  @{

/// def de la map qui fait la liaison entre les string et les énumérés

class ClassPourEnuTypeQuelParticulier
{ public:
  /// def de la map qui fait la liaison entre les string et les énumérés
  static map < string, EnuTypeQuelParticulier , std::less < string> > map_EnuTypeQuelParticulier;
  // def de la grandeur statique qui permet de remplir la map
  static ClassPourEnuTypeQuelParticulier remplir_map;
  // le constructeur qui rempli effectivement la map
  ClassPourEnuTypeQuelParticulier();
  };
/// @}  // end of group

// Retourne le nom du type de grandeur a partir de son identificateur de
// type enumere id_TypeQuelParticulier correspondant
string NomTypeQuelParticulier (EnuTypeQuelParticulier id_TypeQuelParticulier);

// indique si le string correspond à un type quelParticulier reconnu ou non
bool Existe_typeQuelParticulier(string& nom);

// Retourne l'identificateur de type enumere associe au nom du type de grandeur
// nom_TypeQuelParticulier
EnuTypeQuelParticulier Id_nomTypeQuelParticulier (char* nom_TypeQuelParticulier);
EnuTypeQuelParticulier Id_nomTypeQuelParticulier (string& nom_TypeQuelParticulier);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, EnuTypeQuelParticulier& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnuTypeQuelParticulier& a);

#endif
