/*! \file EnumCourbe1D.h
    \brief Enumeration des différentes courbes 1D existantes
* \date      19/01/2001
*/

// FICHIER : EnumCourbe1D.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        19/01/2001                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Enumeration des différentes courbes 1D existantes        *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ENUMCOURBE_1_D_H
#define ENUMCOURBE_1_D_H
#include <iostream>
using namespace std;
#include <string.h>
#include <string>
#include <stdlib.h>

/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des différentes courbes 1D existantes

enum EnumCourbe1D { COURBEPOLYLINEAIRE_1_D=1, COURBE_EXPOAFF, COURBE_UN_MOINS_COS
                    ,CPL1D,COURBEPOLYNOMIALE, F1_ROND_F2, F1_PLUS_F2
                    , F_CYCLIQUE, F_CYCLE_ADD, F_UNION_1D
                    ,COURBE_TRIPODECOS3PHI,COURBE_SIXPODECOS3PHI
                    ,COURBE_POLY_LAGRANGE, COURBE_EXPO_N, COURBE_EXPO2_N
                    ,COURBE_RELAX_EXPO, COURBE_COS, COURBE_SIN, COURBE_TANH
                    ,COURBEPOLYHERMITE_1_D, COURBE_LN_COSH
                    ,COURBE_EXPRESSION_LITTERALE_1D,COURBE_EXPRESSION_LITTERALE_AVEC_DERIVEE_1D
                    , AUCUNE_COURBE1D};
/// @}  // end of group



// Retourne le nom d'une courbe1D a partir de son identificateur de
// type enumere id_Courbe1D correspondant
string Nom_Courbe1D (EnumCourbe1D id_Courbe1D);

// Retourne l'identificateur de type enumere associe au nom d'une courbe1D
EnumCourbe1D Id_Nom_Courbe1D (const string& nom_Courbe1D) ;

// Retourne vrai si le nom passé en argument représente un type de courbe reconnu
// sinon false
bool Type_EnumCourbe1D_existe(const std::string& nom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, EnumCourbe1D& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnumCourbe1D& a);

#endif
