/*! \file EnumTypeGradient.h
    \brief def du gradient de vitesse pour différentes conditions
* \date      28/03/2003
*/

// FICHIER : EnumTypeGradient.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
 *     DATE:        28/03/2003                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Défini une énumération des différents types de calcul du    *
 *     gradient de vitesse.                                             *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
// Afin de realiser un gain en place memoire, et une vérification de type plus aisé qu' avec
// les entiers 


#ifndef ENUM_TYPE_GRADIENT_V_H
#define ENUM_TYPE_GRADIENT_V_H

//#include "Debug.h"
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{


//            ---------- info --------------------------
///   def du gradient de vitesse pour différentes conditions
//
//  GRADVITESSE_V_TDT   : le gradient est déterminé à partir de ddl de vitesse à t+dt
//  GRADVITESSE_VCONST  : calcul en considérant V constant =delta X/delta t dans la config à t+(delta t)/2
//  GRADVITESSE_VT_VTDT : calcul à partir des ddl de vitesse : moyenne de t et t+dt, dans la config
//                        à t+(delta t)/2
 

enum Enum_type_gradient { GRADVITESSE_V_TDT=1, GRADVITESSE_VCONST, GRADVITESSE_VT_VTDT};
/// @}  // end of group

// Retourne le nom du type de gradient a partir de son identificateur de
// type enumere id_ddl correspondant
char* Nom_type_gradient ( Enum_type_gradient id_ddl);

// Retourne l'identificateur de type enumere associe au nom du type de gradient
 Enum_type_gradient Id_nom_type_gradient (char* nom_ddl);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_type_gradient& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_type_gradient& a);

// pour faire de l'inline
#ifndef MISE_AU_POINT
  #include "EnumTypeGradient.cc"
  #define  Enum_type_gradient_deja_inclus
#endif

#endif
