/*! \file EnumTypePilotage.h
    \brief def de l'enuméré concernant les types de pilotage
*/
// FICHIER : EnumTypePilotage.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des types de pilotage sont
// stockes a l'aide d'un type enumere. Les fonctions Nom_TypePilotage et Id_nom_TypePilotage 
// facilitent la liaison entre type énuméré et string


#ifndef ENUM_TYPEPILOTAGE_H
#define ENUM_TYPEPILOTAGE_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de pilotage

enum EnumTypePilotage { PILOTAGE_BASIQUE=1,PILOT_GRADIENT,AUCUN_PILOTAGE};
/// @}  // end of group


// Retourne le nom du type de pilotage a partir de son identificateur de
// type enumere id_TypePilotage correspondant
char* Nom_TypePilotage (const EnumTypePilotage id_TypePilotage);

// Retourne l'identificateur de type enumere associe au nom du type de pilotage
EnumTypePilotage Id_nom_TypePilotage (const char* nom_TypePilotage);

// retourne si la chaine de caractère existe ou pas en tant que EnumTypePilotage
bool Existe_dans_TypePilotage(const char* nom_TypePilotage);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, EnumTypePilotage& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const EnumTypePilotage& a);

#endif
