/*! \file EnumTypeVitesseDefor.h
    \brief Définition de l'enuméré concernant les types de vitesse de déformation
*/
// FICHIER : EnumTypeVitesseDefor.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des différents type de calcul de vitesse
// de déformation  sont stockes a l'aide d'un type enumere. Les fonctions Nom_TypeVitDef et 
// Id_nom_TypeVitDef rendent possible le lien entre les noms des types de vitesse de déformation
// et les identificateurs de type enumere correspondants.


#ifndef ENUM_TYPEVITESSEDEFOR_H
#define ENUM_TYPEVITESSEDEFOR_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de vitesse de déformation

enum Enum_TypeVitDef { D_MOY_EPS_SUR_DT = 1,D_AVEC_V_A_T_PLUS_DT,
   D_AVEC_DX_SUR_DT_A_T_PLUS_DT_SUR2, D_AVEC_V_MOY_A_T_PLUS_DT_SUR2};
/// @}  // end of group


// Retourne un nom de type de vitesse de déformation a partir de son identificateur de
// type enumere id_TypeVitDef correspondant
char* Nom_TypeVitDef (const Enum_TypeVitDef id_TypeVitDef) ;

// Retourne l'identificateur de type enumere associe au nom du type
// de vitesse de déformation nom_TypeVitDef
Enum_TypeVitDef Id_nom_TypeVitDef (const char* nom_TypeVitDef);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_TypeVitDef& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_TypeVitDef& a);


#endif


