/*! \file Enum_IO_XML.h
    \brief Enumération des différentes entree/sortie XML
* \date      20/01/2005
*/

// FICHIER : Enum_IO_XML.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        20/01/2005                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Enumération des différentes entree/sortie XML              *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef ENUM_ENTREE_SORTIE_XML_H
#define ENUM_ENTREE_SORTIE_XML_H

#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Enumération des différentes entree/sortie XML

enum Enum_IO_XML { XML_TYPE_GLOBAUX = 1,XML_IO_POINT_INFO,XML_IO_POINT_BI, XML_IO_ELEMENT_FINI
                   ,XML_ACTION_INTERACTIVE,XML_STRUCTURE_DONNEE};
/// @}  // end of group

const int nombre_maxi_de_type_de_Enum_IO_XML = 6;



// Retourne un nom de type de Enum_IO_XML a partir de son identificateur de
// type enumere id_Enum_IO_XML correspondant
string Nom_Enum_IO_XML (Enum_IO_XML id_Enum_IO_XML);

// Retourne l'identificateur de type enumere associe au nom du type
// de Enum_IO_XML nom_Enum_IO_XML
Enum_IO_XML Id_nom_Enum_IO_XML (string nom_Enum_IO_XML);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_IO_XML& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_IO_XML& a);


#endif


