// FICHIER : Enum_PiPoCo.cc


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


#include "Enum_PiPoCo.h"


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
//#ifdef SYSTEM_MAC_OS_X
//  #include <stringfwd.h> // a priori ce n'est pas portable
//#el
#if defined  SYSTEM_MAC_OS_CARBON
  #include <stringfwd.h> // a priori ce n'est pas portable
#else  
  #include <string.h> // pour le flot en memoire centrale
#endif
#include <string>


// Retourne un nom  a partir de son identificateur de
// type enumere Enum_PiPoCo correspondant
string Nom_Enum_PiPoCo (const Enum_PiPoCo id_Enum)
{	string result;
	switch (id_Enum)
	{
		case NON_PoutrePlaqueCoque : result="NON_PoutrePlaqueCoque"; break;
		case POUTRE : result="POUTRE"; break;
		case PLAQUE : result="PLAQUE"; break;
		case COQUE : result="COQUE"; break;
		default :
			cout << "\nErreur : valeur incorrecte du type Enum_PiPoCo !\n";
			cout << "Nom_Enum_PiPoCo(Enum_PiPoCo ) \n";
			Sortie(1);
	};
	return result;
	
};
						
// Retourne l'identificateur de type enumere associe au nom du type nom_Enum
Enum_PiPoCo Id_Enum_PiPoCo (const string& nom_Enum)
{   Enum_PiPoCo result;
	if ( nom_Enum == "NON_PoutrePlaqueCoque" )
		result=NON_PoutrePlaqueCoque;
	else if ( nom_Enum == "POUTRE" )
		result=POUTRE;
	else if ( nom_Enum == "PLAQUE" )
		result=PLAQUE;
	else if ( nom_Enum == "COQUE" )
		result=COQUE;
	else
	{
		cout << "\nErreur : nom d'identificateur poutre ou plaque ou coque inconnu !\n";
		cout << "Id_Enum_PiPoCo(char* ) \n";
		Sortie(1);
	};
	return result;	
};

// indique le type en fonction de l'interpolation et du découpage
Enum_PiPoCo TypePiPoCo(Enum_interpol id_interpol,Enum_geom id_geom)
{  Enum_PiPoCo retour = NON_PoutrePlaqueCoque;
	  
//enum Enum_interpol { RIEN_INTERPOL = 1,CONSTANT
//           , LINEAIRE, QUADRATIQUE, QUADRACOMPL, CUBIQUE, CUBIQUE_INCOMPL
//	          , LINQUAD, HERMITE, SFE1,SFE2,SFE3, SFE3C
//           , SFE1_3D,SFE2_3D,SFE3_3D, SFE3C_3D, SEG1, BIE1, BIE2};
	  // on ne traite que les cas qui conduise à un retour différent de  NON_PoutrePlaqueCoque
	  switch (id_geom)
    {  case TRIANGLE :
        { //retour = PLAQUE ; // sauf contraire dans la suite, c'est une plaque
          // non c'est une membrane !!
          switch (id_interpol)
          {  case SFE1 : case SFE2 : case SFE3 : case SFE3C :
             case QSFE1: case QSFE3 :
             case SFE1_3D : case SFE2_3D : case SFE3_3D : 
             case SFE3C_3D :
               retour = COQUE; break;
             // maintenant cas d'une membrane
             case LINEAIRE : case QUADRATIQUE : case QUADRACOMPL :
             case CUBIQUE : case CUBIQUE_INCOMPL :
             case LINQUAD : case  HERMITE :
               retour = NON_PoutrePlaqueCoque; break;
             // sinon par défaut message:
             default:
               		cout << "\nErreur : ce cas n'est pas traite !\n"
                      << " id_interpol= " << Nom_interpol(id_interpol)
                      << " , id_geom= " << Nom_geom(id_geom)  ;
		               cout << "\n TypePiPoCo(Enum_interpol id_interpol,Enum_geom id_geom) \n"
                      << endl ;
                 Sortie(1);
          };
         break;
       }
  //		case QUADRANGLE : retour=PLAQUE; break;  non: une membrane
  //		case SEG_AXI : retour=PLAQUE; break;   non une membrane
      case POUT : retour=POUTRE; break;
      case PS1 : retour=POUTRE; break;
      default:
        retour = NON_PoutrePlaqueCoque;
   };
  return retour;
};

// indique si c'est un élément  sfe ou non
bool ElementSfe(Enum_interpol id_interpol)
	{ return  ( (id_interpol >= SFE1) && (id_interpol <= SFE3C_3D) ); 
	};

// surcharge de la lecture
istream & operator >> (istream & entree, Enum_PiPoCo & result)
 { char nom_Enum[35];
   entree >> nom_Enum;
   result = Id_Enum_PiPoCo ( nom_Enum);
   return entree;
 };  
 
    
   // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_PiPoCo& a)
    { // on ecrit la forme caractère
       sort << Nom_Enum_PiPoCo(a) << " ";
       return sort;      
     }; 
	
