/*! \file Enum_PiPoCo.h
    \brief Définir un type énuméré pour la différenciation entre des  éléments classiques et le cas poutre plaque ou coque.
* \date      20/06/2007
*/


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        20/06/2007                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerard.rio@univ-ubs.fr)             *
 *                  Tel 0297874576   fax : 02.97.87.45.72               *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Définir un type énuméré pour la différenciation entre des  *
 *     éléments classiques et le cas poutre plaque ou coque.            *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/



#ifndef ENUM_PIPOCO_H
#define ENUM_PIPOCO_H
#include <iostream>
using namespace std;
#include "Enum_interpol.h"
#include "Enum_geom.h"

/// @addtogroup Group_types_enumeres
///  @{

/// Définir un type énuméré pour la différenciation entre des  éléments classiques et le cas poutre plaque ou coque.

enum Enum_PiPoCo { NON_PoutrePlaqueCoque = 0, POUTRE , PLAQUE, COQUE };
/// @}  // end of group




// Retourne un nom  a partir de son identificateur de
// type enumere Enum_PiPoCo correspondant
string Nom_Enum_PiPoCo (const Enum_PiPoCo id_Enum) ;

// Retourne l'identificateur de type enumere associe au nom du type  nom_Enum
Enum_PiPoCo Id_Enum_PiPoCo (const string& nom_Enum);

// indique si c'est un élément  sfe ou non
bool ElementSfe(Enum_interpol id_interpol);

// indique le type en fonction de l'interpolation et du découpage
Enum_PiPoCo TypePiPoCo(Enum_interpol id_interpol,Enum_geom id_geom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_PiPoCo& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_PiPoCo& a);


#endif

