/*! \file Enum_StabMembrane.h
    \brief Enumeration des différentes méthodes permettant  de stabiliser les membranes transversalement
* \date      30/06/2017
*/


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        30/06/2017                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Enumeration des différentes méthodes permettant          *
 *             de stabiliser les membranes transversalement             *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     * 
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ENUM_STABMEMBRANE_BIEL_H
#define ENUM_STABMEMBRANE_BIEL_H
#include <iostream>
#include <string.h>
#include <string>
using namespace std;  //introduces namespace std
#include <stdlib.h>

/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des différentes méthodes permettant  de stabiliser les membranes transversalement

enum Enum_StabMembraneBiel { STABMEMBRANE_BIEL_NON_DEFINIE = 0
                             ,STABMEMBRANE_BIEL_PREMIER_ITER
                             ,STABMEMBRANE_BIEL_PREMIER_ITER_INCR
                             ,STABMEMBRANE_BIEL_Gerschgorin_PREMIER_ITER
                             ,STABMEMBRANE_BIEL_Gerschgorin_PREMIER_ITER_INCR
                             ,STABMEMBRANE_BIEL_PREMIER_ITER_NORMALE_AU_NOEUD
                             ,STABMEMBRANE_BIEL_PREMIER_ITER_INCR_NORMALE_AU_NOEUD
                             ,STABMEMBRANE_BIEL_Gerschgorin_PREMIER_ITER_NORMALE_AU_NOEUD
                             ,STABMEMBRANE_BIEL_Gerschgorin_PREMIER_ITER_INCR_NORMALE_AU_NOEUD
                             ,STAB_M_B_ITER_1_VIA_F_EXT
                             ,STAB_M_B_ITER_INCR_1_VIA_F_EXT
                             ,STAB_M_B_ITER_1_VIA_F_EXT_NORMALE_AU_NOEUD
                             ,STAB_M_B_ITER_INCR_1_VIA_F_EXT_NORMALE_AU_NOEUD
                           };
/// @}  // end of group



// Retourne le nom a partir de son identificateur de type enumere 
string Nom_StabMembraneBiel (Enum_StabMembraneBiel id_StabMembraneBiel);

// Retourne l'identificateur de type enumere associe au nom d'une StabMembraneBiel
Enum_StabMembraneBiel Id_Enum_StabMembraneBiel(const string& nom_StabMembraneBiel) ;

// Retourne vrai si le nom passé en argument représente un type de StabMembraneBiel reconnu
// sinon false
bool Type_Enum_StabMembraneBiel_existe(const string& nom);

// retourne vrai si le type enumere se termine par _NORMALE_AU_NOEUD
bool Contient_Normale_au_noeud(Enum_StabMembraneBiel id_StabMembraneBiel);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_StabMembraneBiel& a);
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_StabMembraneBiel& a);

#endif
