/*! \file Enum_categorie_loi_comp.h
    \brief def de l'enuméré concernant les booléens ddl.
*/
// FICHIER : Enum_categorie_loi_comp.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, le type de categorie des lois de comportement sont
// stockes a l'aide d'un type enumere. Les fonctions Nom_categorie_comp et Id_nom_categorie_comp 
// rendent possible le lien entre les noms des lois de comportement et les identificateurs
// de type enumere correspondants.


#ifndef ENUM_CATEGORIE_LOI_COMP_H
#define ENUM_CATEGORIE_LOI_COMP_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant catégories de lois de comportement.

// *** important ****, pour chaque groupe de loi il faut que les catégories qui sont les plus
// générales, donc qui englobent les autres, doivent avoir une position plus haute 
// ceci par exemple pour que la fonction Categorie_loi_comp_la_plus_complete fonctionne

enum Enum_categorie_loi_comp { CAT_MECANIQUE=1,CAT_THERMO_MECANIQUE, CAT_THERMO_PHYSIQUE
                               ,CAT_FROTTEMENT
                               ,RIEN_CATEGORIE_LOI_COMP};
/// @}  // end of group


// ***** !!!! penser à changer  *****
//****** nbmax_caractere_Enum_categorie_loi_comp si nécessaire *****
const int nbmax_caractere_Enum_categorie_loi_comp = 26; 

// Retourne le nom d'une loi de comportement a partir de son identificateur de
// type enumere id_categorie_loi_comp correspondant
char* Nom_categorie_loi_comp (const Enum_categorie_loi_comp id_comport);

// Retourne l'identificateur de type enumere associe au nom de la loi de
// comportement nom_comport
Enum_categorie_loi_comp Id_nom_categorie_loi_comp (const char* nom_comport);

// indique si oui ou non la categorie est mécanique c'est-à-dire satisfait à Loi_comp_abstraite
bool GroupeMecanique(const Enum_categorie_loi_comp id_comport);

// indique si oui ou non la categorie est thermique c'est-à-dire satisfait à CompThermoPhysiqueAbstraite
bool GroupeThermique(const Enum_categorie_loi_comp id_comport);

// indique si oui ou non la categorie est frottement (de contact) c-a-d lié à CompFrotAbstraite
bool GroupeFrottement(const Enum_categorie_loi_comp id_comport);

// ramène un énuméré qui correspond à la catégorie la plus complète des deux passés en arguement
// par exemple la catégorie CAT_THERMO_MECANIQUE englobe la catégorie CAT_MECANIQUE
// au paravant on test si les deux sont du même groupe, sinon erreur
Enum_categorie_loi_comp Categorie_loi_comp_la_plus_complete(const Enum_categorie_loi_comp id_comport1,
                                                            const Enum_categorie_loi_comp id_comport2);
// test si deux énumérés correspondent à la même catégorie
bool MemeCategorie(const Enum_categorie_loi_comp id_comport1,const Enum_categorie_loi_comp id_comport2);
// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_categorie_loi_comp& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_categorie_loi_comp& a);

#endif
