/*! \file Enum_chargement.h
    \brief def de l'enuméré concernant les types de chargement
*/
// FICHIER : Enum_chargement.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms de chargement
// sont stockes a l'aide d'un type enumere. Les fonctions Nom_chargement et
// Id_nom_chargement rendent possible le lien entre les noms des types de chargement
// et les identificateurs de type enumere correspondants.


#ifndef ENUM_CHARGEMENT_H
#define ENUM_CHARGEMENT_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de chargement

enum Enum_chargement { RIEN_CHARGEMENT = 1,UNIFORME
           , PRESSION, PONCTUELLE, PRESSDIR, PHYDRO
	          , LINEIQUE, VOLUMIQUE, LINEIC_SUIVEUSE,P_HYDRODYNA,TORSEUR_PONCT};
/// @}  // end of group



// Retourne un nom du chargement a partir de son identificateur de
// type enumere id_chargement correspondant
string Nom_chargement (const Enum_chargement id_chargement) ;

// Retourne l'identificateur de type enumere associe au nom du type
// de chargement nom_chargement
Enum_chargement Id_nom_chargement (const string& nom_chargement);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_chargement& a);
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_chargement& a);


#endif


