/*! \file Enum_comp.h
    \brief def de l'enuméré permettant d'identifier chaque loi de comportement
*/
// FICHIER : Enum_comp.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des lois de comportement sont
// stockes a l'aide d'un type enumere. Les fonctions Nom_comp et Id_nom_comp rendent
// possible le lien entre les noms des lois de comportement et les identificateurs
// de type enumere correspondants.


#ifndef ENUM_COMP_H
#define ENUM_COMP_H
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{


/// énuméré permettant d'identifier chaque loi de comportement
enum Enum_comp { ISOELAS=1,ISOELAS1D, ISOELAS2D_D,ISOELAS2D_C,ISO_ELAS_ESPO1D
                 ,ISO_ELAS_SE1D,ISO_ELAS_ESPO3D
                 ,ORTHOELA3D,ORTHOELA2D_D,ORTHOELA2D_C
                 ,HYPO_ORTHO3D,HYPO_ORTHO2D_D,HYPO_ORTHO2D_C
                 ,PROJECTION_ANISOTROPE_3D
                 ,VISCOELA,ISOHYPER,ISOHYPER1,ISOHYPER10, ISOHYSTE
                 ,TRELOAR, ISOHYPER3DFAVIER1,ISOHYPER3DFAVIER2,ISOHYPER3DFAVIER3
                 ,ISOHYPER3DFAVIER4,ISOHYPER3DORGEAS1,ISOHYPER3DORGEAS2
                 ,ISOHYPERBULK3,ISOHYPERBULK_GENE
                 ,PRANDTL_REUSS,PRANDTL_REUSS2D_D,PRANDTL_REUSS2D_C
                 ,PRANDTL_REUSS1D
                 ,NEWTON1D,NEWTON2D_C,NEWTON2D_D,NEWTON3D
                 ,HYPO_ELAS3D,HYPO_ELAS2D_C,HYPO_ELAS2D_D,HYPO_ELAS1D
                 ,MAXWELL1D,MAXWELL2D_C,MAXWELL2D_D,MAXWELL3D
                 ,LOI_ADDITIVE_EN_SIGMA,LOI_DES_MELANGES_EN_SIGMA,LOI_CRITERE
					            ,LOI_CONTRAINTES_PLANES,LOI_CONTRAINTES_PLANES_DOUBLE,LOI_DEFORMATIONS_PLANES
                 ,HYSTERESIS_1D,HYSTERESIS_3D,HYSTERESIS_BULK,LOI_ISO_THERMO
                 ,MOONEY_RIVLIN_1D,MOONEY_RIVLIN_3D,POLY_HYPER3D,HART_SMITH3D,MAHEO_HYPER
                 ,HYPER_EXTERNE_W
                 ,LOI_DE_TAIT,LOI_VIA_UMAT,LOI_VIA_UMAT_CP
                 ,LOI_COULOMB
                 ,LOI_RIEN1D,LOI_RIEN2D_D,LOI_RIEN2D_C,LOI_RIEN3D
                 ,RIEN_COMP};
/// @}  // end of group


/// @addtogroup Group_types_enumeres
///  @{

/// énuméré permettant de savoir si une loi est : 1D, 2D en deformations planes ou contraintes planes, 3D générale
enum Enum_comp_3D_CP_DP_1D { COMP_1D =1, COMP_CONTRAINTES_PLANES, COMP_DEFORMATIONS_PLANES, COMP_3D , RIEN_COMP_3D_CP_DP_1D}; 					  
/// @}  // end of group


// ***** !!!! penser à changer  *****
//****** nbmax_caractere_Enum_comp si nécessaire *****
const int nbmax_caractere_Enum_comp = 26; 

// ----- 1) concernant Enum_comp ------

// Retourne le nom d'une loi de comportement a partir de son identificateur de
// type enumere id_comport correspondant
string Nom_comp(const Enum_comp id_comport);

// Retourne l'identificateur de type enumere associe au nom de la loi de
// comportement nom_comport
Enum_comp Id_nom_comp(const string& nom_comport);

// indique si la loi est inactive mécaniquement
// typiquement de type : LOI_RIEN... ou RIEN_COMP
bool Loi_rien(const Enum_comp id_comport);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_comp& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_comp& a);

// ----- 2) concernant Enum_comp_3D_CP_DP_1D -----

// Retourne le nom  a partir de son identificateur du type enumere id_Enum_comp_3D_CP_DP_1D correspondant
string Nom_comp_3D_CP_DP_1D(const Enum_comp_3D_CP_DP_1D id_Enum_comp_3D_CP_DP_1D);

// Retourne l'identificateur de type enumere associe à un nom nom_comp_3D_CP_DP_1D
Enum_comp_3D_CP_DP_1D Id_nom_comp_3D_CP_DP_1D(const string nom_comp_3D_CP_DP_1D);

// indique le type Enum_comp_3D_CP_DP_1D correspondant à une  loi de comportement
Enum_comp_3D_CP_DP_1D Comp_3D_CP_DP_1D(const Enum_comp id_comport);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_comp_3D_CP_DP_1D& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_comp_3D_CP_DP_1D& a);


#endif
