/*! \file Enum_dure.h
    \brief Défini une énumération en temps.
* \date      19/01/2001
*/

// FICHIER : Enum_dure.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
 *     DATE:        19/01/2001                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Défini une énumération en temps.                           *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
// Afin de realiser un gain en place memoire, et une vérification de type plus aisé qu' avec
// les entiers 
// 10 carractere maxi


#ifndef ENUM_DURE_H
#define ENUM_DURE_H

//#include "Debug.h"
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Défini une énumération en temps.

enum Enum_dure { TEMPS_0 = 0, TEMPS_t , TEMPS_tdt };
/// @}  // end of group


// Retourne le nom du temps a partir de son identificateur de
// type enumere id_ddl correspondant
string Nom_dure ( Enum_dure id_ddl);

// Retourne l'identificateur de type enumere associe au nom du temps
Enum_dure Id_nom_dure (const string& nom_ddl);

// test si l'identificateur de type enumere associe au nom du temps
// existe bien : ramène true s'il s'agit bien d'un type reconnu
bool Existe_nom_dure (const string& nom_ddl);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_dure& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_dure& a);

// pour faire de l'inline
#ifndef MISE_AU_POINT
  #include "Enum_dure.cc"
  #define  Enum_dure_deja_inclus
#endif

#endif
