/*! \file Enum_geom.h
    \brief Définition de l'enuméré concernant les types de modélisations de géométrie .
*/
// FICHIER : Enum_geom.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms des geometrie des elements sont
// stockes a l'aide d'un type enumere. Les fonctions Nom_geom et Id_nom_geom rendent
// possible le lien entre les noms des geometries et les identificateurs
// de type enumere correspondants.


#ifndef ENUM_GEOM_H
#define ENUM_GEOM_H
#include <iostream>
using namespace std;
#include "Enum_type_geom.h"


/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de modélisations de géométrie .

enum Enum_geom { RIEN_GEOM = 1, TRIANGLE , QUADRANGLE, TETRAEDRE, PENTAEDRE, HEXAEDRE, SEGMENT
                 ,TRIA_AXI,QUAD_AXI,SEG_AXI
                 ,POINT,POINT_CP,POUT,PS1 };
/// @}  // end of group


//*** si on ajoute un élément, il faut penser à complèter l'énumération: Enum_PiPoCo					  

// Retourne le nom d'une geometrie a partir de son identificateur de
// type enumere id_geom correspondant
string Nom_geom (const Enum_geom id_geom);

// Retourne l'identificateur de type enumere associe au nom de geometrie
// nom_geom
Enum_geom Id_nom_geom (const string& nom_geom);

// Retourne le type de géométrie générique: POINT_G, LIGNE, SURFACE, VOLUME,
// associée à l'Enum_geom
Enum_type_geom Type_geom_generique(const Enum_geom id_geom);
// retourne vrai s'il s'agit d'un type axisymétrique
bool TestEnum_geom_axisymetrique(const Enum_geom id_geom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_geom& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_geom& a);

#endif
