/*! \file Enum_liaison_noeud.h
    \brief Enuméré pour définir les différents types de liaison  entre noeuds.
* \date      02/09/2016
*/

// FICHIER : Enum_liaison_noeud.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        02/09/2016                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Enuméré pour définir les différents types de liaison     *
 *             entre noeuds.                                            *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/

// Afin de realiser un gain en place memoire, et une vérification de type plus aisé qu' avec
// les entiers 


#ifndef ENUM_LIAISON_NOEUD_H
#define ENUM_LIAISON_NOEUD_H

//#include "Debug.h"
#include <iostream>
using namespace std;

/// @addtogroup Group_types_enumeres
///  @{

/// Enuméré pour définir les différents types de liaison  entre noeuds.

enum Enum_liaison_noeud { PAS_LIER = 0, LIER_COMPLET };
/// @}  // end of group


// Retourne le nom  a partir de son identificateur de
// type enumere  correspondant
string Nom_liaison_noeud ( Enum_liaison_noeud id_ddl);

// Retourne l'identificateur de type enumere associe au nom
Enum_liaison_noeud Id_nom_liaison_noeud (const string& nom);
 
// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_liaison_noeud& a);
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_liaison_noeud& a);


#endif
