/*! \file Enum_mat.h
    \brief Définition de l'enuméré concernant les types de matériau.
*/
// FICHIER : Enum_mat.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


// Afin de realiser un gain en place memoire, les noms de materiaux sont
// stockes a l'aide d'un type enumere. Les fonctions Nom_mat et Id_nom_mat rendent
// possible le lien entre le nom des materiaux et les identificateurs
// de type enumere correspondants.


#ifndef ENUM_MAT_H
#define ENUM_MAT_H
#include <iostream>
using namespace std;


/// @addtogroup Group_types_enumeres
///  @{

/// Définition de l'enuméré concernant les types de matériau.

enum Enum_mat { ACIER=1, BETON, COMPOSITE };
/// @}  // end of group



// Retourne le nom d'un materiau a partir de son identificateur de
// type enumere id_mater correspondant
char* Nom_mat (Enum_mat id_mater);

// Retourne l'identificateur de type enumere associe au nom de 
// materiau nom_mater
Enum_mat Id_nom_mat (char* nom_mater);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_mat& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_mat& a);

#endif
