/*! \file Enum_Proj_aniso.h
    \brief Enumeration des différentes méthodes concernant les techniques de projection anisotrope
* \date      11/06/2019
*/

// FICHIER : Enum_Proj_aniso.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        11/06/2019                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 *     BUT:    Enumeration des différentes méthodes concernant          *
 *             les techniques de projection anisotrope                  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ENUM_PROJ_ANISO_H
#define ENUM_PROJ_ANISO_H
#include <iostream>
#include <string.h>
#include <string>
using namespace std;  //introduces namespace std
#include <stdlib.h>

/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des différentes méthodes concernant les techniques de projection anisotrope

enum Enum_proj_aniso { AUCUNE_PROJ_ANISO = 0, PROJ_ORTHO };
/// @}  // end of group



// Retourne le nom a partir de son identificateur de type enumere 
string Nom_proj_aniso(Enum_proj_aniso id_proj_aniso);

// Retourne l'identificateur de type enumere associe au nom d'un proj_aniso
Enum_proj_aniso Id_Nom_proj_aniso(const string& nom_proj_aniso) ;

// Retourne vrai si le nom passé en argument représente un type de proj_aniso reconnu
// sinon false
bool Type_Enum_proj_aniso_existe(const string& nom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_proj_aniso& a);
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_proj_aniso& a);

#endif
