/*! \file Enum_Suite.h
    \brief Enumeration des différentes Suites existantes
* \date      19/01/2001
*/

// FICHIER : Enum_Suite.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        19/01/2001                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Enumeration des différentes Suites existantes            *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ENUM_SUITE_H
#define ENUM_SUITE_H
#include <iostream>
#include <string.h>
#include <string>
using namespace std;  //introduces namespace std
#include <stdlib.h>

/// @addtogroup Group_types_enumeres
///  @{

/// Enumeration des différentes Suites existantes

enum Enum_Suite { SUITE_EQUIDISTANTE = 1, SUITE_ARITHMETIQUE, SUITE_GEOMETRIQUE
                  ,SUITE_NON_DEFINIE};
/// @}  // end of group



// Retourne le nom d'une Suite a partir de son identificateur de
// type enumere id_Suite correspondant
char* Nom_Suite (Enum_Suite id_Suite);

// Retourne l'identificateur de type enumere associe au nom d'une Suite
Enum_Suite Id_Nom_Suite (const char* nom_Suite) ;

// Retourne vrai si le nom passé en argument représente un type de suite reconnu
// sinon false
bool Type_Enum_Suite_existe(const string& nom);

// surcharge de l'operator de lecture
istream & operator >> (istream & entree, Enum_Suite& a);  
// surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort, const Enum_Suite& a);

#endif
