
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Def des ddl lie a un noeud d'un element ( different des ddl*
 *           lies aux noeuds globaux).                                  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef DDLNOEUDELEMENT_T
#define DDLNOEUDELEMENT_T
#include "Enum_ddl.h"
#include "Tableau_T.h"
//#ifdef SYSTEM_MAC_OS_X
//  #include <stringfwd.h> // a priori ce n'est pas portable
//#el
#if defined  SYSTEM_MAC_OS_CARBON
  #include <stringfwd.h> // a priori ce n'est pas portable
#else  
  #include <string.h> // pour le flot en memoire centrale
#endif
#include <string>


/// @addtogroup Les_classes_Ddl_en_tout_genre
///  @{
///


///     BUT:    Def des ddl lie a un noeud d'un element ( different des ddl lies aux noeuds globaux)
///
/// \author    Gérard Rio
/// \version   1.0
/// \date       23/01/97
///
///

class DdlNoeudElement
{
   // surcharge de l'operator de lecture
   friend istream & operator >> (istream & entree, DdlNoeudElement& a);  
   // surcharge de l'operator d'ecriture
   friend ostream & operator << (ostream & sort, const DdlNoeudElement& a);

  public :
    // VARIABLES PUBLIQUES :
    
    // CONSTRUCTEURS :
      DdlNoeudElement (); // par defaut
      // def en fonction dunombre de ddl a stocker
      DdlNoeudElement (int n);
      // cas où il y a un seul identificateur de ddl a stocker
      DdlNoeudElement (Enum_ddl e) :
       tb(1,e)
          {} ;
      // de copie 
      DdlNoeudElement (const  DdlNoeudElement& a);
    // DESTRUCTEUR :
      ~DdlNoeudElement ();
    // Methodes
      // surcharge de l'affectation
      DdlNoeudElement& operator = (const DdlNoeudElement & a);
      // surcharge des tests
      bool operator == (const DdlNoeudElement & a) const ;
      bool operator != (const DdlNoeudElement & a) const ;  
    
      // le tableau des identificateurs de ddl
      Tableau <Enum_ddl> tb;
      // pointeur du tableau de connection ddl noeud element/ ddl noeud global
      // pour l'instant ne sert à rien donc en commentaire
   protected :   
    //  int * pt; 
 };
 /// @}  // end of group

#endif  
