// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : CourbePolyLineaire1D_simpli.h
// classe  : CourbePolyLineaire1D_simpli


/************************************************************************
 *     DATE:        03/04/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Cas d'une définition très simplifié d'une  fonction 1D   *
 *             poly-linéaire 1D qui hérite de la fontion poly-linéaire  *
 *             générique. Permet ainsi de minimiser la taille des infos *
 *             à lire.                                                  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef COURBEPOLYLINEAIRE_1_D_SIMPLI_H
#define COURBEPOLYLINEAIRE_1_D_SIMPLI_H

#include "CourbePolyLineaire1D.h"
#include "Tableau_T.h"
#include "Coordonnee2.h"
/// @addtogroup Les_courbes_1D
///  @{
///

/**
*
*     BUT:    Cas d'une définition très simplifié d'une  fonction 1D
*             poly-linéaire 1D qui hérite de la fontion poly-linéaire  
*             générique. Permet ainsi de minimiser la taille des infos
*             à lire.
*
* \author    Gérard Rio
* \version   1.0
* \date      03/04/2004
* \brief       Classe permettant le calcul d'une fonction 1D de type : poly-linéaire 1D simplifiée, qui hérite de la fontion poly-linéaire
*
*/

class Cpl1D : public CourbePolyLineaire1D 
{
  public :

    // CONSTRUCTEURS :
    // par défaut
    
    Cpl1D(string nom = "") : CourbePolyLineaire1D(nom,CPL1D) {};
    
    // de copie
    Cpl1D(const Cpl1D& Co) : CourbePolyLineaire1D(Co) {};
    Cpl1D(const Courbe1D& Co);
    
    // DESTRUCTEUR :
    ~Cpl1D() {};
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_courbes(const string& nom, UtilLecture * );
    
    // def info fichier de commande
    void Info_commande_Courbes1D(UtilLecture & entreePrinc);
    // sortie du schemaXML: en fonction de enu 
    void SchemaXML_Courbes1D(ofstream& sort,const Enum_IO_XML enu) ;

 };
 /// @}  // end of group
#endif  
