// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

#include "Courbe_expo_n.h"
#include "Sortie.h"
#include "ConstMath.h"
#include "MathUtil.h"
#include "ParaGlob.h"

    // CONSTRUCTEURS :
Courbe_expo_n::Courbe_expo_n(string nom) :
 Courbe1D(nom,COURBE_EXPO_N)
 ,alpha(-ConstMath::tresgrand),xn(1.),gamma(0.) 
 // alpha négatif -> pb que l'on peut détecter pour savoir si c'est complet
  {};
    
    // de copie
Courbe_expo_n::Courbe_expo_n(const Courbe_expo_n& Co) :
 Courbe1D(Co)
 ,alpha(Co.alpha),xn(Co.xn),gamma(Co.gamma)
  {};
    // de copie à partir d'une instance générale
Courbe_expo_n::Courbe_expo_n(const Courbe1D& Coo) :
 Courbe1D(Coo)
  { if (Coo.Type_courbe() != COURBE_EXPO_N)
      { cout << "\n erreur dans le constructeur de copie pour une courbe COURBE_EXPO_N "
             << " à partir d'une instance générale ";
        cout << "\n Courbe_expo_n::Courbe_expo_n(const Courbe1D& Co) ";
           Sortie(1);          
          };
    // définition des données
    Courbe_expo_n & Co = (Courbe_expo_n&) Coo;
    alpha = Co.alpha; xn = xn;gamma = Co.gamma;       
  };

    // DESTRUCTEUR :
Courbe_expo_n::~Courbe_expo_n() 
  {};
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
void Courbe_expo_n::Affiche() const 
  { cout << "\n Courbe_expo_n: nom_ref= " << nom_ref;
    cout << "\n alpha=" << alpha << " n= " << xn << " gamma= " << gamma;
   };

    // vérification que tout est ok, pres à l'emploi
    // ramène true si ok, false sinon
bool Courbe_expo_n::Complet_courbe()const
 { bool ret = Complet_var(); // on regarde du coté de la classe mère tout d'abord
   // puis les variables propres
   if (alpha == -ConstMath::tresgrand) ret = false;
   if (!ret && (ParaGlob::NiveauImpression() >0))
      { cout << "\n ***** la courbe n'est pas complete ";
        this->Affiche();
        };
   return ret;
  } ;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
void Courbe_expo_n::LectDonnParticulieres_courbes(const string& nom,UtilLecture * entreePrinc)
  {  if (nom == "")  { *(entreePrinc->entree) >> nom_ref;}
     else {nom_ref=nom;};
     entreePrinc->NouvelleDonnee();  // lecture d'une nouvelle ligne
     // on lit l'entête de chaque coef et on vérifie
     string nomcoef;
     // lecture de gamma
     *(entreePrinc->entree) >> nomcoef >> gamma;
     if(nomcoef != "gamma=")
         { cout << "\n erreur en lecture du coefficient gamma , on attendait le mot cle: gamma="
                << " et on a lue : " << nomcoef;
           entreePrinc->MessageBuffer("**Courbe_expo_n::LectureDonneesParticulieres**");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          };
     // lecture de alpha
     *(entreePrinc->entree) >> nomcoef >> alpha;
     if(nomcoef != "alpha=")
         { cout << "\n erreur en lecture du coefficient alpha , on attendait le mot cle: alpha="
                << " et on a lue : " << nomcoef;
           entreePrinc->MessageBuffer("**Courbe_expo_n::LectureDonneesParticulieres**");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          };
     // lecture de n
     *(entreePrinc->entree) >> nomcoef >> xn;
     if(nomcoef != "n=")
         { cout << "\n erreur en lecture du coefficient gamma , on attendait le mot cle: n="
                << " et on a lue : " << nomcoef;
           entreePrinc->MessageBuffer("**Courbe_expo_n::LectureDonneesParticulieres**");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          };
   };
    
// def info fichier de commande
void Courbe_expo_n::Info_commande_Courbes1D(UtilLecture & entreePrinc) 
  {  
     ofstream & sort = *(entreePrinc.Commande_pointInfo()); // pour simplifier
     sort << "\n#............................................"
          << "\n# exemple de definition d'une courbe COURBE_EXPO_N ( f(x) = (gamma + alpha * x)**n )|" 
          << "\n#"
          << "\n courbe_monte   COURBE_EXPO_N  # nom de la courbe puis le type de la courbe"
          << "\n      # def des coeff de la courbe COURBE_EXPO_N "
          << "\n      gamma= 10. alpha= -2. n= 1.3"
          << endl;
   };
        
    // ramène la valeur 
double Courbe_expo_n::Valeur(double x)  
  { return pow((gamma+alpha * x),xn);
  };

    // ramène la valeur et la dérivée en paramètre
Courbe1D::ValDer Courbe_expo_n::Valeur_Et_derivee(double x)  
  {  ValDer ret; // def de la valeur de retour
    //tout d'abord on regarde s'il est en dehors des bornes
    ret.valeur = pow((gamma+alpha * x),xn);
    ret.derivee = alpha*xn*pow((gamma+alpha * x),xn-1);
    return ret;
   };
    
    // ramène la dérivée 
double Courbe_expo_n::Derivee(double x)  
  {  return ( alpha*xn*pow((gamma+alpha * x),xn-1));
   };
        
    // ramène la valeur et les dérivées première et seconde en paramètre
Courbe1D::ValDer2 Courbe_expo_n::Valeur_Et_der12(double x)
  {  ValDer2 ret; // def de la valeur de retour
    ret.valeur = pow((gamma+alpha * x),xn);
    ret.derivee = alpha*xn*pow((gamma+alpha * x),xn-1);
    ret.der_sec = alpha*alpha*xn*(xn-1.)*pow((gamma+alpha * x),xn-2);
    return ret;
   };
        
    // ramène la dérivée seconde
double Courbe_expo_n::Der_sec(double x)
  {  return alpha*alpha*xn*(xn-1.)*pow((gamma+alpha * x),xn-2);
   };
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
Courbe1D::Valbool  Courbe_expo_n::Valeur_stricte(double x)  
  { Valbool ret; // def de la valeur de retour
    ret.valeur = pow((gamma+alpha * x),xn);
    ret.dedans = true;
    return ret;
   }; 
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
Courbe1D::ValDerbool  Courbe_expo_n::Valeur_Et_derivee_stricte(double x)  
  { ValDerbool ret; // def de la valeur de retour
    ret.valeur = pow((gamma+alpha * x),xn);
    ret.derivee = alpha*xn*pow((gamma+alpha * x),xn-1);
    ret.dedans = true;
    return ret;
   }; 
    
	//----- lecture écriture de restart -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
void Courbe_expo_n::Lecture_base_info(ifstream& ent,const int cas)
  { // on n'a que des grandeurs constantes
    if (cas == 1)
      { string nom;
        // lecture et vérification de l'entête
        ent >> nom;
        if (nom != "Courbe_expo_n")
         { cout << "\n erreur dans la vérification du type de courbe lue ";    
           cout << "\n Courbe_expo_n::Lecture_base_info(... ";
           Sortie(1); 
           } 
        // lecture des infos
        ent >> nom >> gamma >> nom >> alpha >> nom >> xn ;
       }
   };

    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
void Courbe_expo_n::Ecriture_base_info(ofstream& sort,const int cas)
  { // on n'a que des grandeurs constantes
    if (cas == 1)
      { sort << " Courbe_expo_n ";
        sort << "  gamma= " << gamma  << " alpha= " << alpha <<" n= " << xn ;
       }
   };
         
// sortie du schemaXML: en fonction de enu 
void Courbe_expo_n::SchemaXML_Courbes1D(ofstream& ,const Enum_IO_XML enu)
  {
	switch (enu)
	{ case XML_TYPE_GLOBAUX :
		{
		 break;
		}
		case XML_IO_POINT_INFO :
		{
		 break;
		}
		case XML_IO_POINT_BI :
		{
		 break;
		}
		case XML_IO_ELEMENT_FINI :
		{
		 break;
		}
	};		
  };
     
