// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : SuiteReel.h
// classe  : SuiteReel


/************************************************************************
 *     DATE:        14/11/2007                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant des calculs relatifs à des suites reel *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                $     *
 ************************************************************************/

 
#ifndef SUITE_M_H
#define SUITE_M_H

#include "Enum_Suite.h"

/** @defgroup def_classes_suites_reel def_classes_suites_reel
*
* \author    Gérard Rio
* \version   1.0
* \date      14/11/2007
* \brief       Classe permettant des calculs relatifs à des suites reel
*
*/

/// @addtogroup def_classes_suites_reel
///  @{
///

/// classe d'interface (virtuelle pure) pour la création et l'utilisation de suite
class SuiteReel
{
  public :

    // CONSTRUCTEURS :
    // par défaut
    SuiteReel(  Enum_Suite typ = SUITE_NON_DEFINIE) 
        : typeSuite(typ) {};
    // de copie
    SuiteReel(const SuiteReel& Co) : typeSuite(Co.typeSuite) {};
    // DESTRUCTEUR :
    virtual ~SuiteReel(){};
    
    // METHODES PUBLIQUES :
    
    // affichage de la courbe
	virtual void Affiche() const = 0 ;

    // ramène la valeur d'un élément n de la suite
    virtual double U_n(int n)  =0 ;
    
    // ramène la somme de la suite de 0 à n en considérant la suite:
    // 0.  U_0   U_1  U_2 etc 
    // ici = sum_0^n U_i
    // de manière arbitraire pour n=-1 ==> 0
    // pour n < -1 --> erreur
    virtual double Somme_Suite(int n) = 0;
        
    // ramène le type de la suite
    Enum_Suite Type_Suite() const { return typeSuite;};
    
    // interactif écran-clavier pour saisir les paramètres d'une suite
    // si amplitude != 0 alors il faut n non nulle et:
    // on a Somme_suite(n) = amplitude, ce qui permet pour les suites de réduire le nombre de paramètre à lire
    virtual void Def_suite(double amplitude, int n) =0;
     
  protected :
    
    // VARIABLES PROTEGEES :
    Enum_Suite typeSuite; // type de la suite
    
    
 };
 /// @}  // end of group
 
#endif  
