// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : Suite_equidistante.h
// classe  : Suite_equidistante


/************************************************************************
 *     DATE:        14/11/2007                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant des calculs relatifs à des suites      *
 *             equidistante: u_n=U_(n-1)                                *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                $     *
 ************************************************************************/

#ifndef SUITE_EQUIDISTANTE_M_H
#define SUITE_EQUIDISTANTE_M_H

#include "SuiteReel.h"

/// @addtogroup def_classes_suites_reel
///  @{
///

/// Suite_equidistante: Classe permettant des calculs relatifs à des suites equidistante: u_n=U_(n-1)
class Suite_equidistante : public SuiteReel
{
  public :

    // CONSTRUCTEURS :
    // par défaut
    Suite_equidistante( ) 
        : SuiteReel(SUITE_EQUIDISTANTE), U_0(0.) {};
    // de copie
    Suite_equidistante(const Suite_equidistante& Co) 
         : SuiteReel(Co), U_0(Co.U_0) {};
    // DESTRUCTEUR :
    ~Suite_equidistante(){};
    
    // METHODES PUBLIQUES :
    
    // affichage de la courbe
	void Affiche() const ;
    // ramène la valeur d'un élément n de la suite
    double U_n(int ) { return U_0; };
    // ramène la somme de la suite de 0 à n
    // de manière arbitraire pour n=-1 ==> 0
    // pour n < -1 --> erreur
    double Somme_Suite(int n) ;
    
    // si amplitude != 0 alors il faut n non nulle et:
    // on a U_n=amplitude, ce qui permet pour les suites de réduire le nombre de paramètre à lire
    void Def_suite(double amplitude, int n) ;
     
  protected :
    
    // VARIABLES PROTEGEES :
    double U_0 ; // la valeur du terme courant de la suite
    
 };
 /// @}  // end of group

#endif  
