

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


# include <iostream>
using namespace std;  //introduces namespace std
#include <stdlib.h>
#include "Sortie.h"
#include "NevezTenseur.h"
#include "Tenseur1.h"
#include "Tenseur2.h"
#include "Tenseur3.h"

TenseurHH * NevezTenseurHH(int dim,double val)
  { TenseurHH * ptr;
    switch (dim)
     { case -1 : ptr = new Tenseur1HH(val); break;
       case 1 : ptr = new Tenseur1HH(val); break;
       case -2 : ptr = new Tenseur_ns2HH(val); break;
       case 2 : ptr = new Tenseur2HH(val); break;
       case -3 : ptr = new Tenseur_ns3HH(val); break;
       case 3 : ptr = new Tenseur3HH(val); break;
       default : { cout << " erreur d'allocation de nouveau tenseurHH \n"; Sortie(1); }
     }
    return ptr; 
   };    
TenseurBB * NevezTenseurBB(int dim,double val)
  { TenseurBB * ptr;
    switch (dim)
     { case -1 : ptr = new Tenseur1BB(val); break;
       case 1 : ptr = new Tenseur1BB(val); break;
       case -2 : ptr = new Tenseur_ns2BB(val); break;
       case 2 : ptr = new Tenseur2BB(val); break;
       case -3 : ptr = new Tenseur_ns3BB(val); break;
       case 3 : ptr = new Tenseur3BB(val); break;
       default : { cout << " erreur d'allocation de nouveau tenseurBB \n"; Sortie(1); }
     }
    return ptr;  
   };  
TenseurHB * NevezTenseurHB(int dim,double val)
  { TenseurHB * ptr;
    switch (dim)
     { case 1 : ptr = new Tenseur1HB(val); break;
       case 2 : ptr = new Tenseur2HB(val); break;
       case 3 : ptr = new Tenseur3HB(val); break;
       default : { cout << " erreur d'allocation de nouveau tenseurHB \n"; Sortie(1); }
     }
    return ptr;    
   };  
TenseurBH * NevezTenseurBH(int dim,double val)
  { TenseurBH * ptr;
    switch (dim)
     { case 1 : ptr = new Tenseur1BH(val); break;
       case 2 : ptr = new Tenseur2BH(val); break;
       case 3 : ptr = new Tenseur3BH(val); break;
       default : { cout << " erreur d'allocation de nouveau tenseurBH \n"; Sortie(1); }
     }
    return ptr;     
   };   

TenseurHH * NevezTenseurHH(const TenseurHH& a)
  { TenseurHH * ptr;
    switch (a.Dimension())
     { case -1 : ptr = new Tenseur1HH((Tenseur1HH &) a); break;
       case 1 : ptr = new Tenseur1HH((Tenseur1HH &) a); break;
       case -2 : ptr = new Tenseur_ns2HH((Tenseur_ns2HH &) a); break;
       case 2 : ptr = new Tenseur2HH((Tenseur2HH &) a); break;
       case -3 : ptr = new Tenseur_ns3HH((Tenseur_ns3HH &) a); break;
       case 3 : ptr = new Tenseur3HH((Tenseur3HH &) a); break;
       default : { cout << " erreur d'allocation de nouveau tenseurHH \n"; Sortie(1); }
     }
    return ptr;     
   };   
        
TenseurBB * NevezTenseurBB(const TenseurBB& a)
  { TenseurBB * ptr;
    switch (a.Dimension())
     { case -1 : ptr = new Tenseur1BB((Tenseur1BB&) a); break;
       case 1 : ptr = new Tenseur1BB((Tenseur1BB&) a); break;
       case -2 : ptr = new Tenseur_ns2BB((Tenseur_ns2BB&) a); break;
       case 2 : ptr = new Tenseur2BB((Tenseur2BB&) a); break;
       case -3 : ptr = new Tenseur_ns3BB((Tenseur_ns3BB&) a); break;
       case 3 : ptr = new Tenseur3BB((Tenseur3BB&) a); break;
       default : { cout << " erreur d'allocation de nouveau tenseurBB \n"; Sortie(1); }
     }
    return ptr;  
   };  
TenseurHB * NevezTenseurHB(const TenseurHB& a)
  { TenseurHB * ptr;
    switch (a.Dimension())
     { case 1 : ptr = new Tenseur1HB((Tenseur1HB&) a); break;
       case 2 : ptr = new Tenseur2HB((Tenseur2HB&) a); break;
       case 3 : ptr = new Tenseur3HB((Tenseur3HB&) a); break;
       default : { cout << " erreur d'allocation de nouveau tenseurHB \n"; Sortie(1); }
     }
    return ptr;    
   };  
TenseurBH * NevezTenseurBH(const TenseurBH& a)
  { TenseurBH * ptr;
    switch (a.Dimension())
     { case 1 : ptr = new Tenseur1BH((Tenseur1BH&) a); break;
       case 2 : ptr = new Tenseur2BH((Tenseur2BH&) a); break;
       case 3 : ptr = new Tenseur3BH((Tenseur3BH&) a); break;
       default : { cout << " erreur d'allocation de nouveau tenseurBH \n"; Sortie(1); }
     }
    return ptr;     
   };   

// définition de tenseur d'ordre 2 à partir d'un produit tensoriel de vecteur
// en sortie le programme retourne le pointeur affecte sur un tenseur de
// la bonne taille en fonction de la dimension du pb
TenseurBB * Produit_tensorielBB(const CoordonneeB & aB, const CoordonneeB & bB)
 {switch (aB.Dimension())
     { case 1 : return (&(Tenseur1BB::Prod_tensoriel(aB,bB))); break;
       case 2 : return (&(Tenseur2BB::Prod_tensoriel(aB,bB))); break;
       case 3 : return (&(Tenseur3BB::Prod_tensoriel(aB,bB))); break;
       default : { cout << " erreur de definition du Produit_tensorielBB \n"; Sortie(1); }
     }
   return NULL;
  };
TenseurHH * Produit_tensorielHH(const CoordonneeH & aH, const CoordonneeH & bH)
 {switch (aH.Dimension())
     { case 1 : return (&(Tenseur1HH::Prod_tensoriel(aH,bH))); break;
       case 2 : return (&(Tenseur2HH::Prod_tensoriel(aH,bH))); break;
       case 3 : return (&(Tenseur3HH::Prod_tensoriel(aH,bH))); break;
       default : { cout << " erreur de definition du Produit_tensorielHH \n"; Sortie(1); }
     }
   return NULL;
  };
TenseurHB * Produit_tensorielHB(const CoordonneeH & aH, const CoordonneeB & bB)
 {switch (aH.Dimension())
     { case 1 : return (&(Tenseur1HB::Prod_tensoriel(aH,bB))); break;
       case 2 : return (&(Tenseur2HB::Prod_tensoriel(aH,bB))); break;
       case 3 : return (&(Tenseur3HB::Prod_tensoriel(aH,bB))); break;
       default : { cout << " erreur de definition du Produit_tensorielHB \n"; Sortie(1); }
     }
   return NULL;
  };
TenseurBH * Produit_tensorielBH(const CoordonneeB & aB, const CoordonneeH & bH)
 {switch (aB.Dimension())
     { case 1 : return (&(Tenseur1BH::Prod_tensoriel(aB,bH))); break;
       case 2 : return (&(Tenseur2BH::Prod_tensoriel(aB,bH))); break;
       case 3 : return (&(Tenseur3BH::Prod_tensoriel(aB,bH))); break;
       default : { cout << " erreur de definition du Produit_tensorielBH \n"; Sortie(1); }
     }
   return NULL;
  };
