

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "ConstMath.h" 
#include "MathUtil.h"
//#include "Debug.h"
#include "Tenseur1_TroisSym.h"
#include "Tenseur1.h"
#include "CharUtil.h"


#ifndef  TenseurQ1_TroisSym_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes HHHH
//------------------------------------------------------------------   

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_HHHH::TenseurQ1_troisSym_HHHH() :  
 ipointe() // par défaut
  { dimension = 106;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_HHHH::TenseurQ1_troisSym_HHHH( const double& x1111):  
 ipointe()  
  { dimension = 106;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=x1111;
  };
   
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_HHHH::~TenseurQ1_troisSym_HHHH() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_HHHH::TenseurQ1_troisSym_HHHH ( const TenseurHHHH & B):  
 ipointe() 
  { listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);  
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_HHHH::TenseurQ1_troisSym_HHHH (  const TenseurQ1_troisSym_HHHH & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    this->t[0] = B.t[0];
  };        

    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ1_troisSym_HHHH::Inita(double val)   
	{t[0]=val;};

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ1_troisSym_HHHH::operator + ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 106) Message(1,"TenseurQ1_troisSym_HHHH::operator + ( etc..");
     #endif
     res =  new TenseurQ1_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] + B.t[0]; //somme des données
    return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ1_troisSym_HHHH::operator += ( const TenseurHHHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_HHHH::operator += ( etc..");
     #endif
     this->t[0] += B.t[0];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ1_troisSym_HHHH::operator - () const 
  {  TenseurHHHH * res;
     res =  new TenseurQ1_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = - this->t[0]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ1_troisSym_HHHH::operator - ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_HHHH::operator - ( etc..");
     #endif
     res =  new TenseurQ1_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] - B.t[0]; //soustraction des données
     return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1_troisSym_HHHH::operator -= ( const TenseurHHHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_HHHH::operator -= ( etc..");
     #endif
     this->t[0] -= B.t[0];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ1_troisSym_HHHH::operator = ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_HHHH::operator = ( etc..");
     #endif
     this->t[0] = B.t[0];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
     return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ1_troisSym_HHHH::operator * ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ1_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * b; //multiplication des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1_troisSym_HHHH::operator *= ( const double & b)
  {this->t[0] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ1_troisSym_HHHH::operator / ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ1_troisSym_HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1_troisSym_HHHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     res->t[0] = this->t[0] / b; //division des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1_troisSym_HHHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1_troisSym_HHHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     this->t[0] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& TenseurQ1_troisSym_HHHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 1) 
           Message(1,"TenseurQ1_troisSym_HHHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     res =  new Tenseur1HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1BB & a1BB = *((Tenseur1BB*) &aBB); // passage en dim 1
     res->Coor(1,1)=t[0] * a1BB(1,1);
     return *res ;
  };


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ1_troisSym_HHHH::Transpose1et2avec3et4() const   
   { TenseurHHHH * res;
     res =  new TenseurQ1_troisSym_HHHH(*this);
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ1_troisSym_HHHH::Affectation_trans_dimension(const TenseurHHHH & aHHHH,bool plusZero)
   { switch (abs(aHHHH.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aHHHH(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHHH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ1_troisSym_HHHH::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ1_troisSym_HHHH::operator == ( const TenseurHHHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_HHHH::operator == ( etc..");
     #endif
     if (this->t[0] != B.t[0]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ1_troisSym_HHHH::Change (int i, int j, int k, int l, const double& val) 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1_troisSym_HHHH::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }      
   #endif
   t[0] = val; 
  }; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ1_troisSym_HHHH::ChangePlus (int i, int j, int k, int l, const double& val)
 {
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1))
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1_troisSym_HHHH::ChangePlus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }
   #endif
   t[0] += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ1_troisSym_HHHH::operator () (int i, int j, int k, int l) const 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n double  TenseurQ1_troisSym_HHHH::operator () (int i, int j, int k, int l) const ";
        Sortie(1);
       }      
   #endif
   return t[0];
  }; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ1_troisSym_HHHH::MaxiComposante() const
  { return DabsMaxiTab(t,1) ;
   };

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ1_troisSym_HHHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1_troisSym_HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> this->t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ1_troisSym_HHHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ1_troisSym_HHHH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[0] << " ";
    return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ1_troisSym_HHHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 106) A.Message(1,"operator >> (istream & entree, TenseurQ1_troisSym_HHHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1_troisSym_HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> A.t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ1_troisSym_HHHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ1_troisSym_HHHH ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[0] << " ";
    return sort;      
  };
 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_BBBB::TenseurQ1_troisSym_BBBB() :  
 ipointe() // par défaut
  { dimension = 106;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_BBBB::TenseurQ1_troisSym_BBBB( const double& x1111):  
 ipointe()  
  { dimension = 106;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=x1111;
  };
  
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_BBBB::~TenseurQ1_troisSym_BBBB() 
{ listdouble1.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_BBBB::TenseurQ1_troisSym_BBBB ( const TenseurBBBB & B):  
 ipointe() 
  { listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    t[0]=B(1,1,1,1);
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ1_troisSym_BBBB::TenseurQ1_troisSym_BBBB (  const TenseurQ1_troisSym_BBBB & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble1.push_front(Reels1());  // allocation
    ipointe = listdouble1.begin(); // recup de la position de la maille dans la liste
    t = &((ipointe)->donnees);    // recup de la position des datas dans la maille
    this->t[0] = B.t[0];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ1_troisSym_BBBB::Inita(double val)   
	{t[0]=val;};

#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ1_troisSym_BBBB::operator + ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 106) Message(1,"TenseurQ1_troisSym_BBBB::operator + ( etc..");
     #endif
     res =  new TenseurQ1_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] + B.t[0]; //somme des données
     return *res ;};
    
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ1_troisSym_BBBB::operator += ( const TenseurBBBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_BBBB::operator += ( etc..");
     #endif
     this->t[0] += B.t[0];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ1_troisSym_BBBB::operator - () const 
  {  TenseurBBBB * res;
     res =  new TenseurQ1_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = - this->t[0]; //oppose
     return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ1_troisSym_BBBB::operator - ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_BBBB::operator - ( etc..");
     #endif
     res =  new TenseurQ1_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] - B.t[0]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1_troisSym_BBBB::operator -= ( const TenseurBBBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_BBBB::operator -= ( etc..");
     #endif
     this->t[0] -= B.t[0];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ1_troisSym_BBBB::operator = ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_BBBB::operator = ( etc..");
     #endif
     this->t[0] = B.t[0];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
     return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ1_troisSym_BBBB::operator * ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ1_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     res->t[0] = this->t[0] * b; //multiplication des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1_troisSym_BBBB::operator *= ( const double & b)
  { this->t[0] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ1_troisSym_BBBB::operator / ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ1_troisSym_BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1_troisSym_BBBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     res->t[0] = this->t[0] / b; //division des données
     return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ1_troisSym_BBBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ1_troisSym_BBBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     this->t[0] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ1_troisSym_BBBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 1) 
           Message(1,"TenseurQ1_troisSym_BBBB::operator && ( const TenseurBB & aHH)");
     #endif
     TenseurBB * res;
     // deux cas suivant que le tenseur aHH est symétrique ou pas
     // cependant due aux 3 symétries, le résultat est quand même toujours symétrique
     res =  new Tenseur1BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur1HH & a1HH = *((Tenseur1HH*) &aHH); // passage en dim 1
     res->Coor(1,1)=t[0] * a1HH (1,1);
     return *res ;
  };
  

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ1_troisSym_BBBB::Transpose1et2avec3et4() const   
   { TenseurBBBB * res;
     res =  new TenseurQ1_troisSym_BBBB(*this);
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     return *res;
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ1_troisSym_BBBB::Affectation_trans_dimension(const TenseurBBBB & aBBBB,bool plusZero)
   { switch (abs(aBBBB.Dimension()))
       { case 33 : case 22 : case 11 : case 106: case 206: case 306 :
         case 30 : case 10 :
           // ensuite on affecte
           t[0] = aBBBB(1,1,1,1);
           break;
         default:
          Message(1,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBBB.Dimension()))
                    +"n'est pas prise en compte \n Tenseur1BBBB::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ1_troisSym_BBBB::operator == ( const TenseurBBBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 106) Message(1,"TenseurQ1_troisSym_BBBB::operator == ( etc..");
     #endif
     if (this->t[0] != B.t[0]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ1_troisSym_BBBB::Change (int i, int j, int k, int l, const double& val) 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1_troisSym_BBBB::Change (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }      
   #endif
   t[0] = val; 
  }; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ1_troisSym_BBBB::ChangePlus (int i, int j, int k, int l, const double& val)
 {
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1))
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n void TenseurQ1_troisSym_BBBB::ChangePlus (int i, int j, int k, int l, const double& val) ";
        Sortie(1);
       }
   #endif
   t[0] += val;
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ1_troisSym_BBBB::operator () (int i, int j, int k, int l) const 
 { 
   #ifdef MISE_AU_POINT
    if ( (i!=1)||(j!=1)||(k!=1)||(l!=1)) 
      { cout << "\n erreur d'adressage de composante d'un tenseur 1D, demande de ("<<i<<","<<j<<","<<k<<","<<l<<")"
             << "\n double  TenseurQ1_troisSym_BBBB::operator () (int i, int j, int k, int l) const ";
        Sortie(1);
       }      
   #endif
   return t[0];
  }; 
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ1_troisSym_BBBB::MaxiComposante() const
  { return DabsMaxiTab(t,1) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ1_troisSym_BBBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1_troisSym_BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> this->t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ1_troisSym_BBBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ1_troisSym_BBBB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[0] << " ";
    return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ1_troisSym_BBBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 106) A.Message(1,"operator >> (istream & entree, TenseurQ1_troisSym_BBBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ1_troisSym_BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    entree >> A.t[0];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ1_troisSym_BBBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ1_troisSym_BBBB ";
    // puis les datas
    sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[0] << " ";
    return sort;      
  };

 
  
#endif
