

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ2gene.h"
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur2.h"
#include "CharUtil.h"

#ifndef  TenseurQ2gene_H_deja_inclus



///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
//?????????????????????????????????,a faire les divers cas de produit tensoriel de tenseur du quatrieme ordre
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,


// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes 2HHHH
//------------------------------------------------------------------   

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHHH::TenseurQ2geneHHHH() :
 ipointe() // par défaut
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHHH::TenseurQ2geneHHHH( const double val) :
 ipointe() 
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=val;
  };
  
    // cas = 1      : produit tensoriel normal 
    //                  *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    // cas = 2      : produit tensoriel barre
    //                  *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
    // cas = 3      : produit tensoriel under barre
    //                  *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHHH::TenseurQ2geneHHHH(int cas, const TenseurHH & aHH, const TenseurHH & bHH) :
 ipointe() 
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un premier tenseur non symétriques \n",
                   "TenseurQ2geneHHHH::TenseurQ2geneHHHH(bool normal, const"),
                    " TenseurHH & aHH, const TenseurHH & bHH);"));
       if (Dabs(b2HH.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un second tenseur non symétriques \n",
                   "TenseurQ2geneHHHH::TenseurQ2geneHHHH(bool normal, const"),
                    " TenseurHH & aHH, const TenseurHH & bHH);"));
    #endif
    switch (cas)
     { case 1 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
               // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2HH(i,j) * b2HH(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2HH(i,k) * b2HH(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2HH(i,l) * b2HH(j,k);
           break;   
         }
       default :
        { Message(2,"TenseurQ2geneHHHH::TenseurQ2geneHHHH(int cas, const TenseurHH & aHH, const TenseurHH & bHH)");
          Sortie(1);
         }
      }    
  }; 
 
// idem pour des instances de tenseurs 2
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHHH::TenseurQ2geneHHHH(int cas, const Tenseur2HH & a2HH, const Tenseur2HH & b2HH) :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    switch (cas)
     { case 1 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
               // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2HH(i,j) * b2HH(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2HH(i,k) * b2HH(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2HH(i,l) * b2HH(j,k);
           break;   
         }
       default :
        { Message(2,"TenseurQ2geneHHHH::TenseurQ2geneHHHH(int cas, const Tenseur2HH & a2HH, const Tenseur2HH & b2HH)");
          Sortie(1);
         }
      }    
  };      

    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHHH::~TenseurQ2geneHHHH()
{ listdouble16.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHHH::TenseurQ2geneHHHH ( const TenseurHHHH & B) :
 ipointe()
  { dimension = B.dimension;
    #ifdef MISE_AU_POINT
    if (Dabs(dimension) != 20)
      { cout << "\n erreur de dimension, elle devrait etre = 20 ";
        cout << "\n TenseurQ2geneHHHH::TenseurQ2geneHHHH ( TenseurHHHH &) " << endl;
        Sortie(1);
      }  
    #endif
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       t[i] = B.t[i];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHHH::TenseurQ2geneHHHH (  const TenseurQ2geneHHHH & B):
 ipointe()
  { this->dimension = B.dimension;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ2geneHHHH::Inita(double val)
	{ for (int i=0;i< 16;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2geneHHHH::operator + ( const TenseurHHHH & B) const
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 20) Message(2,"TenseurQ2geneHHHH::operator + ( etc..");
     #endif
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ2geneHHHH::operator += ( const TenseurHHHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHHH::operator += ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2geneHHHH::operator - () const
  {  TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2geneHHHH::operator - ( const TenseurHHHH & B) const
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHHH::operator - ( etc..");
     #endif
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneHHHH::operator -= ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHHH::operator -= ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2geneHHHH::operator = ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHHH::operator = ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2geneHHHH::operator * ( const double & b) const
  {  TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneHHHH::operator *= ( const double & b)
  {for (int i = 0; i< 16; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2geneHHHH::operator / ( const double & b) const
  {  TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneHHHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneHHHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneHHHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 16; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& TenseurQ2geneHHHH::operator && ( const TenseurBB & aBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 2)
           Message(2,"TenseurQ2geneHHHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<=2;i++) for (int j=1;j<=i;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) += t[8*i+4*j+2*k+l-15] * a2BB(k,l);
     return *res ;
  };

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurHHBB& TenseurQ2geneHHHH::operator && ( const TenseurBBBB & aBBBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHHH::Prod_gauche( const TenseurBBBB & F)");
     #endif
     TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBBB & a2BBBB = *((TenseurQ2geneBBBB*) &aBBBB); // passage en dim 2
     for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       for(int e=1;e<=2;e++) for (int f=1;f<=2;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBBB.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurHHHH& TenseurQ2geneHHHH::operator && ( const TenseurBBHH & aBBHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHHH::Prod_gauche( const TenseurBBHH & F)");
     #endif
     TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBHH & a2BBHH = *((TenseurQ2geneBBHH*) &aBBHH); // passage en dim 2
     for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       for(int e=1;e<=2;e++) for (int f=1;f<=2;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche 
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre deux
TenseurHH& TenseurQ2geneHHHH::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 2)
           Message(2,"TenseurQ2geneHHHH::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<=2;i++) for (int j=1;j<=i;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) +=  a2BB(k,l) * t[8*k+4*l+2*i+j-15] ;
     return *res ;
  };
  
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurBBHH& TenseurQ2geneHHHH::Prod_gauche( const TenseurBBBB & aBBBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHHH::Prod_gauche( const TenseurBBBB & F)");
     #endif
     TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBBB & a2BBBB = *((TenseurQ2geneBBBB*) &aBBBB); // passage en dim 2
     for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       for(int e=1;e<=2;e++) for (int f=1;f<=2;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBBB.t[8*i+4*j+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHBB
TenseurHHHH& TenseurQ2geneHHHH::Prod_gauche( const TenseurHHBB & aHHBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHHH::Prod_gauche( const TenseurHHBB & F)");
     #endif
     TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHBB & a2HHBB = *((TenseurQ2geneHHBB*) &aHHBB); // passage en dim 2
     for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       for(int e=1;e<=2;e++) for (int f=1;f<=2;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHBB.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
//=========== fin fonction protected ======================	

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  TenseurQ2geneHHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)
  { TenseurHHHH * res;
    res =  new TenseurQ2geneHHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneHHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneHHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2HH(i,j) * b2HH(k,l);
    return *res;                         
  };      
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  TenseurQ2geneHHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH)
  { TenseurHHHH * res;
    res =  new TenseurQ2geneHHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneHHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneHHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2HH(i,k) * b2HH(j,l);
    return *res;                         
  };      

#ifndef MISE_AU_POINT
  inline 
#endif
    // *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
TenseurHHHH &  TenseurQ2geneHHHH::Prod_tensoriel_under_barre(const TenseurHH & aHH, const TenseurHH & bHH)
  { TenseurHHHH * res;
    res =  new TenseurQ2geneHHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneHHHH::Prod_tensoriel_under_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneHHHH::Prod_tensoriel_under_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2HH(i,l) * b2HH(j,k);
    return *res;                         
  };      

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ2geneHHHH::Transpose1et2avec3et4() const
   { TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
       res->t[8*k+4*l+2*i+j-15] = t[8*i+4*j+2*k+l-15];
     return *res;
   };           
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ2geneHHHH::Affectation_trans_dimension(const TenseurHHHH & aHHHH,bool plusZero)
   { switch (abs(aHHHH.Dimension()))
       { case 33 : case 30 : case 306 : case 22 :  case 206:
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[8*k+4*l+2*i+j-15] = aHHHH(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHHH(1,1,1,1);
           break;
         default:
          Message(2,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHHH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ2geneHHHH::Affectation_trans_dimension(");
       };
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // création d'un tenseur symétrique / au deux premiers indices et / au deux derniers indices
    // B(i,i,i,i) = A(i,i,i,i); B(i,j,k,k) = 1/2(A(i,j,k,k)+ A(j,i,k,k)); si 2 premiers indices différents
    // B(i,i,k,l) = 1/2(A(i,i,k,l)+ A(j,i,l,k)); si 2 derniers indices différents 
    // B(i,j,k,l) = 1/4(A(i,j,k,l)+ A(j,i,k,l) + A(i,j,l,k)+ A(j,i,l,k)); si tous les indices différents
TenseurHHHH & TenseurQ2geneHHHH::Symetrise1et2_3et4() const
	{TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // en fait si l'on veut  faire avec les tests il y a plus de travail en test qu'en calcul car le
     // le troisième cas est le plus courant donc on ne met pas de test et on divise par 4 tout le temps
     for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
       res->t[8*i+4*j+2*k+l-15] = 0.25 * (t[8*i+4*j+2*k+l-15] + t[8*j+4*i+2*k+l-15]
                                           + t[8*i+4*j+2*l+k-15] + t[8*j+4*i+2*l+k-15]);
     return *res;
	};

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ2geneHHHH::operator == ( const TenseurHHHH & B) const
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHHH::operator == ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ2geneHHHH::Change (int i, int j, int k, int l, const double& val)
  { t[8*i+4*j+2*k+l-15]  = val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ2geneHHHH::ChangePlus (int i, int j, int k, int l, const double& val)
  { t[8*i+4*j+2*k+l-15]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ2geneHHHH::operator () (int i, int j, int k, int l) const
 { return t[8*i+4*j+2*k+l-15]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ2geneHHHH::MaxiComposante() const
  { return DabsMaxiTab(t,16) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ2geneHHHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneHHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ2geneHHHH::Ecriture(ostream & sort) const
  { // écriture du type
    sort << "TenseurQ2geneHHHH ";
    // puis les datas
     for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ2geneHHHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 20) A.Message(2,"operator >> (istream & entree, TenseurQ2geneHHHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneHHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ2geneHHHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ2geneHHHH ";
    // puis les datas
    for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------


    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBBB::TenseurQ2geneBBBB() :
 ipointe() // par défaut
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBBB::TenseurQ2geneBBBB( const double val)  :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel avec 3 cas
    // cas = 1      : produit tensoriel normal 
    //                  *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    // cas = 2      : produit tensoriel barre
    //                  *this=aBB(i,k).bBB(j,l) gHi gHj gHk gHl
    // cas = 3      : produit tensoriel under barre
    //                  *this=aBB(i,l).bBB(j,k) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBBB::TenseurQ2geneBBBB(int cas, const TenseurBB & aBB, const TenseurBB & bBB) :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un premier tenseur non symétriques \n",
                   "TenseurQ2geneBBBB::TenseurQ2geneBBBB(bool normal, const"),
                    " TenseurBB & aBB, const TenseurBB & bBB);"));
       if (Dabs(b2BB.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un second tenseur non symétriques \n",
                   "TenseurQ2geneBBBB::TenseurQ2geneBBBB(bool normal, const"),
                    " TenseurBB & aBB, const TenseurBB & bBB);"));
    #endif
    switch (cas)
     { case 1 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
               // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2BB(i,j) * b2BB(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2BB(i,k) * b2BB(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<=2;i++) for (int j=1;j<=2;j++) for (int k=1;k<=2;k++) for (int l=1;l<=2;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2BB(i,l) * b2BB(j,k);
           break;   
         }
       default :
        { Message(2,"TenseurQ2geneBBBB::TenseurQ2geneBBBB(int cas, const TenseurBB & aBB, const TenseurBB & bBB)");
          Sortie(1);
         }
      }    
  };
//arrêt du changement de 3 en 2 **** à continuer !!
// idem mais avec des instances de tenseur 2
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBBB::TenseurQ2geneBBBB(int cas, const Tenseur2BB & a2BB, const Tenseur2BB & b2BB) :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    switch (cas)
     { case 1 :
        { for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
               // ((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2BB(i,j) * b2BB(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2BB(i,k) * b2BB(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
              // (((i-1)2+(j-1))2+(k-1))2+l-1
              t[8*i+4*j+2*k+l-15] = a2BB(i,l) * b2BB(j,k);
           break;   
         }
       default :
        { Message(2,"TenseurQ2geneBBBB::TenseurQ2geneBBBB(int cas, const Tenseur2BB & a2BB, const Tenseur2BB & b2BB)");
          Sortie(1);
         }
      }    
  };      

    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBBB::~TenseurQ2geneBBBB()
{ listdouble16.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBBB::TenseurQ2geneBBBB ( const TenseurBBBB & B) :
 ipointe()
  { dimension = B.dimension;
    #ifdef MISE_AU_POINT
    if (Dabs(dimension) != 20)
      { cout << "\n erreur de dimension, elle devrait etre = 20 ";
        cout << "\n TenseurQ2geneBBBB::TenseurQ2geneBBBB ( TenseurBBBB &) " << endl;
        Sortie(1);
      }  
    #endif
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       t[i] = B.t[i];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBBB::TenseurQ2geneBBBB (  const TenseurQ2geneBBBB & B) :
 ipointe()
  { this->dimension = B.dimension;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ2geneBBBB::Inita(double val)
	{ for (int i=0;i< 16;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2geneBBBB::operator + ( const TenseurBBBB & B) const
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 20) Message(2,"TenseurQ2geneBBBB::operator + ( etc..");
     #endif
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ2geneBBBB::operator += ( const TenseurBBBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBBB::operator += ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2geneBBBB::operator - () const
  {  TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2geneBBBB::operator - ( const TenseurBBBB & B) const
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBBB::operator - ( etc..");
     #endif
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneBBBB::operator -= ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBBB::operator -= ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2geneBBBB::operator = ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBBB::operator = ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2geneBBBB::operator * ( const double & b) const
  {  TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneBBBB::operator *= ( const double & b)
  {for (int i = 0; i< 16; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2geneBBBB::operator / ( const double & b) const
  {  TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneBBBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneBBBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneBBBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 16; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ2geneBBBB::operator && ( const TenseurHH & aHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 2)
           Message(2,"TenseurQ2geneBBBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<3;i++) for (int j=1;j<=i;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) += t[8*i+4*j+2*k+l-15] * a2HH(k,l);
     return *res ;
  };
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHBB
TenseurBBBB& TenseurQ2geneBBBB::operator && ( const TenseurHHBB & aHHBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBBB::Prod_gauche( const TenseurHHBB & F)");
     #endif
     TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHBB & a2HHBB = *((TenseurQ2geneHHBB*) &aHHBB); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHBB.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurBBHH& TenseurQ2geneBBBB::operator && ( const TenseurHHHH & aHHHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBBB::Prod_gauche( const TenseurHHHH & F)");
     #endif
     TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHHH & a2HHHH = *((TenseurQ2geneHHHH*) &aHHHH); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ2geneBBBB::Prod_gauche( const TenseurHH & aHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 2)
           Message(2,"TenseurQ2geneBBBB::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<3;i++) for (int j=1;j<i;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) +=  a2HH(k,l) * t[8*k+4*l+2*i+j-15] ;
     return *res ;
  };
  
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurBBBB& TenseurQ2geneBBBB::Prod_gauche( const TenseurBBHH & aBBHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBBB::Prod_gauche( const TenseurBBHH & F)");
     #endif
     TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBHH & a2BBHH = *((TenseurQ2geneBBHH*) &aBBHH); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurHHBB& TenseurQ2geneBBBB::Prod_gauche( const TenseurHHHH & aHHHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBBB::Prod_gauche( const TenseurHHHH & F)");
     #endif
     TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHHH & a2HHHH = *((TenseurQ2geneHHHH*) &aHHHH); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
//=========== fin fonction protected ======================	
   
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  TenseurQ2geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)
  { TenseurBBBB * res;
    res =  new TenseurQ2geneBBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2BB(i,j) * b2BB(k,l);
    return *res;                         
  };      
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // *this=aBB(i,k).bBB(j,l) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  TenseurQ2geneBBBB::Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurBB & bBB)
  { TenseurBBBB * res;
    res =  new TenseurQ2geneBBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2BB(i,k) * b2BB(j,l);
    return *res;                         
  };      



#ifndef MISE_AU_POINT
  inline 
#endif
    // *this=aBB(i,l).bBB(j,k) gHi gHj gHk gHl
TenseurBBBB &  TenseurQ2geneBBBB::Prod_tensoriel_under_barre(const TenseurBB & aBB, const TenseurBB & bBB)
  { TenseurBBBB * res;
    res =  new TenseurQ2geneBBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneBBBB::Prod_tensoriel_under_barre(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneBBBB::Prod_tensoriel_under_barre(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2BB(i,l) * b2BB(j,k);
    return *res;                         
  };      

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ2geneBBBB::Transpose1et2avec3et4() const
   { TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
       res->t[8*k+4*l+2*i+j-15] = t[8*i+4*j+2*k+l-15];
     return *res;
   };           

#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ2geneBBBB::Affectation_trans_dimension(const TenseurBBBB & aBBBB,bool plusZero)
   { switch (abs(aBBBB.Dimension()))
       { case 33 : case 30 : case 306 : case 22 :  case 206:
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[8*k+4*l+2*i+j-15] = aBBBB(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBBB(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBBB.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ3geneBBBB::Affectation_trans_dimension(");
       };
   };


#ifndef MISE_AU_POINT
  inline 
#endif
    // création d'un tenseur symétrique / au deux premiers indices et / au deux derniers indices
    // B(i,i,i,i) = A(i,i,i,i); B(i,j,k,k) = 1/2(A(i,j,k,k)+ A(j,i,k,k)); si 2 premiers indices différents
    // B(i,i,k,l) = 1/2(A(i,i,k,l)+ A(j,i,l,k)); si 2 derniers indices différents 
    // B(i,j,k,l) = 1/4(A(i,j,k,l)+ A(j,i,k,l) + A(i,j,l,k)+ A(j,i,l,k)); si tous les indices différents
TenseurBBBB & TenseurQ2geneBBBB::Symetrise1et2_3et4() const
	{TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // en fait si l'on veut  faire avec les tests il y a plus de travail en test qu'en calcul car le
     // le troisième cas est le plus courant donc on ne met pas de test et on divise par 4 tout le temps
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       res->t[8*i+4*j+2*k+l-15] = 0.25 * (t[8*i+4*j+2*k+l-15] + t[8*j+4*i+2*k+l-15]
                                           + t[8*i+4*j+2*l+k-15] + t[8*j+4*i+2*l+k-15]);
     return *res;
	};

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ2geneBBBB::operator == ( const TenseurBBBB & B) const
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBBB::operator == ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ2geneBBBB::Change (int i, int j, int k, int l,const double& val)
  { t[8*i+4*j+2*k+l-15]  = val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ2geneBBBB::ChangePlus (int i, int j, int k, int l,const double& val)
  { t[8*i+4*j+2*k+l-15]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ2geneBBBB::operator () (int i, int j, int k, int l) const
 { return t[8*i+4*j+2*k+l-15]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ2geneBBBB::MaxiComposante() const
  { return DabsMaxiTab(t,  16) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ2geneBBBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneBBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ2geneBBBB::Ecriture(ostream & sort) const
  { // écriture du type
    sort << "TenseurQ2geneBBBB ";
    // puis les datas
     for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ2geneBBBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 20) A.Message(2,"operator >> (istream & entree, TenseurQ2geneBBBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneBBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ2geneBBBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ2geneBBBB ";
    // puis les datas
    for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

 
  
#endif
