

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
*            UNIVERSITE DE BRETAGNE SUD (UBS) --- LORIENT              *
************************************************************************
*           IRDL - Equipe DE GENIE MECANIQUE ET MATERIAUX (EG2M)       *
* Centre de Recherche Rue de Saint Maudé - 56325 Lorient cedex         *
* tel. 02.97.32.82.47                 http://IRDL.fr                   *
************************************************************************
 *     DATE:       16/12/2019                                           *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerard.rio@univ-ubs.fr)             *
 *                  phone  02.97.32.82.47                               *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   Definition d'une classe derivee de tenseur du 4ieme ordre *
 *            de dimension2, il s'agit ici d'une classe générale, sans  *
 *            particularités: c-a-d 16 composantes.                     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef TENSEURQ2GENE_H
#define TENSEURQ2GENE_H

#include <iostream> 
#include "TenseurQ.h"  
#include "PtTabRel.h"
# include "Tableau2_T.h"
#include "Tenseur2.h"

//**************************************************************************
//    pour l'instant on n'utilise que des tenseurs d'ordre deux symétriques
//    à chaque fois qu'apparaît un tenseurs du second ordre
//    si besoin est on améliora
////////////:::://////////////::::::///////////::::::://////////::::::///////

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes 2HHHH
//------------------------------------------------------------------   
class TenseurQ2geneHHHH : public TenseurHHHH
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, TenseurQ2geneHHHH &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const TenseurQ2geneHHHH &);
 
  public :
    // Constructeur
    TenseurQ2geneHHHH() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    TenseurQ2geneHHHH(const double val);
    // initialisation à partir d'un produit tensoriel avec 3 cas
    // cas = 1      : produit tensoriel normal 
    //                  *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    // cas = 2      : produit tensoriel barre
    //                  *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
    // cas = 3      : produit tensoriel under barre
    //                  *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
    TenseurQ2geneHHHH(int cas, const TenseurHH & aHH, const TenseurHH & bHH);
    TenseurQ2geneHHHH(int cas, const Tenseur2HH & aHH, const Tenseur2HH & bHH);

    // DESTRUCTEUR :
    ~TenseurQ2geneHHHH() ;
    // constructeur a partir d'une instance non differenciee  
    TenseurQ2geneHHHH (const  TenseurHHHH &);
    // constructeur de copie  
    TenseurQ2geneHHHH (const  TenseurQ2geneHHHH &);
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurHHHH & operator + ( const TenseurHHHH &) const ;
    void operator += ( const TenseurHHHH &);
    TenseurHHHH & operator - () const ;  // oppose du tenseur
    TenseurHHHH & operator - ( const TenseurHHHH &) const ;
    void  operator -= ( const TenseurHHHH &);
    TenseurHHHH & operator = ( const TenseurHHHH &);
    TenseurHHHH & operator = ( const TenseurQ2geneHHHH & B)
       { return this->operator=((TenseurHHHH &) B); };
    TenseurHHHH & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurHHHH & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit deux fois contracte à droite avec un tenseur du second ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    TenseurHH& operator && ( const TenseurBB & )  const ;
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche 
    TenseurHHHH& operator && ( const TenseurBBHH & )  const ;
    TenseurHHBB& operator && ( const TenseurBBBB & )  const ;

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    static TenseurHHHH &  Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH) ;
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
    static TenseurHHHH &  Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH) ;
    //fonctions définissant le produit tensoriel under_barre de deux tenseurs
    // *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
    static TenseurHHHH &  Prod_tensoriel_under_barre(const TenseurHH & aHH, const TenseurHH & bHH) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurHHHH & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurHHHH & B,bool plusZero);
 
    // création d'un tenseur symétrique / au deux premiers indices et / au deux derniers indices
    // B(i,i,i,i) = A(i,i,i,i); B(i,j,k,k) = 1/2(A(i,j,k,k)+ A(j,i,k,k)); si 2 premiers indices différents
    // B(i,i,k,l) = 1/2(A(i,i,k,l)+ A(j,i,l,k)); si 2 derniers indices différents 
    // B(i,j,k,l) = 1/4(A(i,j,k,l)+ A(j,i,k,l) + A(i,j,l,k)+ A(j,i,l,k)); si tous les indices différents
    TenseurHHHH & Symetrise1et2_3et4() const;
      
    // test
    int operator == ( const TenseurHHHH &) const ;
    
    // change la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data 
	   listdouble16Iter ipointe;
	
	   // fonction pour le produit contracté à gauche
    TenseurHH& Prod_gauche( const TenseurBB & F) const;   
    TenseurBBHH& Prod_gauche( const TenseurBBBB & F) const;   
    TenseurHHHH& Prod_gauche( const TenseurHHBB & F) const;   
  }; 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------
class TenseurQ2geneBBBB : public TenseurBBBB
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, TenseurQ2geneBBBB &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const TenseurQ2geneBBBB &);
 
  public :
    // Constructeur
    TenseurQ2geneBBBB() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    TenseurQ2geneBBBB(const double val);
    // initialisation à partir d'un produit tensoriel avec 3 cas
    // cas = 1      : produit tensoriel normal 
    //                  *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    // cas = 2      : produit tensoriel barre
    //                  *this=aBB(i,k).bBB(j,l) gHi gHj gHk gHl
    // cas = 3      : produit tensoriel under barre
    //                  *this=aBB(i,l).bBB(j,k) gHi gHj gHk gHl
    TenseurQ2geneBBBB(int cas, const TenseurBB & aBB, const TenseurBB & bBB);
    TenseurQ2geneBBBB(int cas, const Tenseur2BB & aBB, const Tenseur2BB & bBB);

    // DESTRUCTEUR :
    ~TenseurQ2geneBBBB() ;
    // constructeur a partir d'une instance non differenciee  
    TenseurQ2geneBBBB (const  TenseurBBBB &);
    // constructeur de copie  
    TenseurQ2geneBBBB (const  TenseurQ2geneBBBB &);
    
   // METHODES PUBLIQUES :
    //2)    virtuelles
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurBBBB & operator + ( const TenseurBBBB &) const ;
    void operator += ( const TenseurBBBB &);
    TenseurBBBB & operator - () const ;  // oppose du tenseur
    TenseurBBBB & operator - ( const TenseurBBBB &) const ;
    void  operator -= ( const TenseurBBBB &);
    TenseurBBBB & operator = ( const TenseurBBBB &);
    TenseurBBBB & operator = ( const TenseurQ2geneBBBB & B)
       { return this->operator=((TenseurBBBB &) B); };
    TenseurBBBB & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurBBBB & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit deux fois contracte à droite avec un tenseur du second ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    TenseurBB& operator && ( const TenseurHH & )  const ;
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    TenseurBBBB& operator && ( const TenseurHHBB & )  const ;
    TenseurBBHH& operator && ( const TenseurHHHH & )  const ;
    
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    static TenseurBBBB &  Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB) ;
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // *this=aBB(i,k).bBB(j,l) gHi gHj gHk gHl
    static TenseurBBBB &  Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurBB & bBB) ;
    //fonctions définissant le produit tensoriel under_barre de deux tenseurs
    // *this=aBB(i,l).bBB(j,k) gHi gHj gHk gHl
    static TenseurBBBB &  Prod_tensoriel_under_barre(const TenseurBB & aBB, const TenseurBB & bBB) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurBBBB & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurBBBB & B,bool plusZero);
 
    // création d'un tenseur symétrique / au deux premiers indices et / au deux derniers indices
    // B(i,i,i,i) = A(i,i,i,i); B(i,j,k,k) = 1/2(A(i,j,k,k)+ A(j,i,k,k)); si 2 premiers indices différents
    // B(i,i,k,l) = 1/2(A(i,i,k,l)+ A(j,i,l,k)); si 2 derniers indices différents 
    // B(i,j,k,l) = 1/4(A(i,j,k,l)+ A(j,i,k,l) + A(i,j,l,k)+ A(j,i,l,k)); si tous les indices différents
    TenseurBBBB & Symetrise1et2_3et4() const;
      
    // test
    int operator == ( const TenseurBBBB &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	   listdouble16Iter ipointe;
	
	   // fonction pour le poduit contracté à gauche
    TenseurBB& Prod_gauche( const TenseurHH & F) const;
    TenseurHHBB& Prod_gauche( const TenseurHHHH & F) const;
    TenseurBBBB& Prod_gauche( const TenseurBBHH & F) const;
  }; 

//  
//------------------------------------------------------------------
//          cas des composantes mixte 2BBHH
//------------------------------------------------------------------
      
class TenseurQ2geneBBHH : public TenseurBBHH
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, TenseurQ2geneBBHH &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const TenseurQ2geneBBHH &);
 
  public :
    // Constructeur
    TenseurQ2geneBBHH() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    TenseurQ2geneBBHH(const double val);
    // initialisation à partir d'un produit tensoriel normal
    //                  *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl    
    TenseurQ2geneBBHH(const TenseurBB & aBB, const TenseurHH & bHH);
    TenseurQ2geneBBHH(const Tenseur2BB & aBB, const Tenseur2HH & bHH);

    // DESTRUCTEUR :
    ~TenseurQ2geneBBHH() ;
    // constructeur a partir d'une instance non differenciee  
    TenseurQ2geneBBHH (const  TenseurBBHH &);
    // constructeur de copie  
    TenseurQ2geneBBHH (const  TenseurQ2geneBBHH &);
    
   // METHODES PUBLIQUES :
    //2)    virtuelles
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurBBHH & operator + ( const TenseurBBHH &) const ;
    void operator += ( const TenseurBBHH &);
    TenseurBBHH & operator - () const ;  // oppose du tenseur
    TenseurBBHH & operator - ( const TenseurBBHH &) const ;
    void  operator -= ( const TenseurBBHH &);
    TenseurBBHH & operator = ( const TenseurBBHH &);
    TenseurBBHH & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurBBHH & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit deux fois contracte à droite avec un tenseur du second ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche 
    TenseurBB& operator && ( const TenseurBB & )  const ;
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    TenseurBBBB& operator && ( const TenseurBBBB & )  const ;
    TenseurBBHH& operator && ( const TenseurBBHH & )  const ;
        
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
    static TenseurBBHH &  Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurHHBB & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurBBHH & B,bool plusZero);
 
    // test
    int operator == ( const TenseurBBHH &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
    void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	   listdouble16Iter ipointe;
	    
	   // fonction pour le poduit contracté à gauche
    TenseurHH& Prod_gauche( const TenseurHH & F) const;   
    TenseurBBHH& Prod_gauche( const TenseurBBHH & F) const;   
    TenseurHHHH& Prod_gauche( const TenseurHHHH & F) const;   
};	   
//  
//------------------------------------------------------------------
//          cas des composantes mixte 2HHBB
//------------------------------------------------------------------
      
class TenseurQ2geneHHBB : public TenseurHHBB
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, TenseurQ2geneHHBB &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const TenseurQ2geneHHBB &);
 
  public :
    // Constructeur
    TenseurQ2geneHHBB() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    TenseurQ2geneHHBB(const double val);

    // initialisation à partir d'un produit tensoriel normal
    //                  *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl    
    TenseurQ2geneHHBB(const TenseurHH & aHH, const TenseurBB & bBB);
    TenseurQ2geneHHBB(const Tenseur2HH & aHH, const Tenseur2BB & bBB);

    // DESTRUCTEUR :
    ~TenseurQ2geneHHBB() ;
    // constructeur a partir d'une instance non differenciee  
    TenseurQ2geneHHBB (const  TenseurHHBB &);
    // constructeur de copie  
    TenseurQ2geneHHBB (const  TenseurQ2geneHHBB &);
    
   // METHODES PUBLIQUES :
    //2)    virtuelles
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurHHBB & operator + ( const TenseurHHBB &) const ;
    void operator += ( const TenseurHHBB &);
    TenseurHHBB & operator - () const ;  // oppose du tenseur
    TenseurHHBB & operator - ( const TenseurHHBB &) const ;
    void  operator -= ( const TenseurHHBB &);
    TenseurHHBB & operator = ( const TenseurHHBB &);
    TenseurHHBB & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurHHBB & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit deux fois contracte à droite avec un tenseur du second ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    TenseurHH& operator && ( const TenseurHH & )  const ;
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    TenseurHHHH& operator && ( const TenseurHHHH & )  const ;
    TenseurHHBB& operator && ( const TenseurHHBB & )  const ;

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
    static TenseurHHBB &  Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurBBHH & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    void Affectation_trans_dimension(const TenseurHHBB & B,bool plusZero);
 
    // test
    int operator == ( const TenseurHHBB &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	   listdouble16Iter ipointe;
	
	   // fonction pour le produit contracté à gauche
    TenseurBB& Prod_gauche( const TenseurBB & F) const;
    TenseurHHBB& Prod_gauche( const TenseurHHBB & F) const;
    TenseurBBBB& Prod_gauche( const TenseurBBBB & F) const;
};	   	   
     
#ifndef MISE_AU_POINT
  #include "TenseurQ2gene-1.cc"
  #include "TenseurQ2gene-2.cc"
  #define  TenseurQ2gene_H_deja_inclus
#endif
    
 
#endif  
