

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ-3.h"
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur3.h"
#include "CharUtil.h"


#ifndef  TenseurQ3_H_deja_inclus
   
// 1) pour le passage repère absolu en repère local
//  T1 et T2 sont des tenseurs, Base représente une base
// pour des tenseurs T1 et T2 symétriques HHHH ou BBBB      
template <class T1,class T2,class Base>  
 T1&  produit44(T1& A,T2& Nous, const Base & gi)
    {   int dim = 3;
        if ((A.Dimension()!=33)|(Nous.Dimension()!=33))
        	{cout << "\n erreur la methode n'est valide que pour la dimension 3"
        	      << "\n ici pour des tenseurs Tenseur3HHHH ou Tenseur3BBBB"
        	      << "\n T1&  produit44(T1& A,T2& Nous, const Base & gi)";
        	 Sortie(1);     
        	};
        int dimint = 3;
        for (int i=1;i<= dim; i++)
          for (int j=1; j<= i; j++)
            for (int k=1;k<= dim; k++)
              for (int l=1; l<= k; l++)
               {int ijkl = (Tenseur3HHHH::cdex3HHHH.odVect(i,j)-1)*6+Tenseur3HHHH::cdex3HHHH.odVect(k,l)-1; 
                 A.t[ijkl]=0.;
                 for (int al=1; al<= dimint; al++)
                   for (int be=1; be<= dimint; be++)
                    for (int cl=1; cl<= dimint; cl++)
                     for (int de=1; de<= dimint; de++)
                     	{ int abcd=(Tenseur3HHHH::cdex3HHHH.odVect(al,be)-1)*6+Tenseur3HHHH::cdex3HHHH.odVect(cl,de)-1; 
                     	  A.t[ijkl] += Nous.t[abcd]*gi(i)(al)*gi(j)(be)
                                      *gi(k)(cl)*gi(l)(de);	
                     	}
                 }       
        return A;      
      };

// 2) pour le changement de base (même dim = 3 qu'actuelle)
//  T1 et T2 sont des tenseurs, Base représente une base
// pour des tenseurs T1 et T2 symétriques HHHH ou BBBB      
template <class T1,class T2,class Base>  
 T1&  produit55(T1& A,T2& Nous, const Base & gi)
    {   int dim = 3;
        if ((A.Dimension()!=33)|(Nous.Dimension()!=33))
        	{cout << "\n erreur la methode n'est valide que pour la dimension 3"
        	      << "\n ici pour des tenseurs Tenseur3HHHH ou Tenseur3BBBB"
        	      << "\n T1&  produit44(T1& A,T2& Nous, const Base & gi)";
        	 Sortie(1);     
        	};
        int dimint = 3;
        for (int i=1;i<= dim; i++)
          for (int j=1; j<= i; j++)
            for (int k=1;k<= dim; k++)
              for (int l=1; l<= k; l++)
               {int ijkl = (Tenseur3HHHH::cdex3HHHH.odVect(i,j)-1)*6+Tenseur3HHHH::cdex3HHHH.odVect(k,l)-1; 
                 A.t[ijkl]=0.;
                 for (int al=1; al<= dimint; al++)
                   for (int be=1; be<= dimint; be++)
                    for (int cl=1; cl<= dimint; cl++)
                     for (int de=1; de<= dimint; de++)
                     	{ int abcd=(Tenseur3HHHH::cdex3HHHH.odVect(al,be)-1)*6+Tenseur3HHHH::cdex3HHHH.odVect(cl,de)-1; 
                     	A.t[ijkl] += Nous.t[abcd]*gi(al)(i)*gi(be)(j)
                                      *gi(cl)(k)*gi(de)(l);	
                     	}
                 }       
        return A;      
      };

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes 3HHHH
//------------------------------------------------------------------   
//    typiquement le produit tensoriel de deux tenseurs symétriques :
//    A(i,j,k,l) = B(i,j) * C(k,l)
//    Chaque tenseur symétrique comporte 6 composantes
// B(1,1)->1, B(2,2)->2, B(3,3)->3, B(2,1)->4, B(3,2)->5, B(3,1)->6
//  du coup A est rangé dans un tableau 6x6

	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHHH::ChangementIndex::ChangementIndex() :
  idx_i(6),idx_j(6),odVect(3)
  { idx_i(1)=1;idx_i(2)=2;idx_i(3)=3;idx_i(4)=2;idx_i(5)=3;idx_i(6)=3;
    idx_j(1)=1;idx_j(2)=2;idx_j(3)=3;idx_j(4)=1;idx_j(5)=2;idx_j(6)=1;
    odVect(1,1)=1;odVect(1,2)=4;odVect(1,3)=6;
    odVect(2,1)=4;odVect(2,2)=2;odVect(2,3)=5;
    odVect(3,1)=6;odVect(3,2)=5;odVect(3,3)=3;
  }; 

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHHH::Tenseur3HHHH() :  
 ipointe() // par défaut
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHHH::Tenseur3HHHH( const double val) :  
 ipointe() 
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    // booleen = false : produit tensoriel barre
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHHH::Tenseur3HHHH(bool normal, const TenseurHH & aHH, const TenseurHH & bHH) :  
 ipointe()
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                    +"Tenseur3HHHH::Tenseur3HHHH(bool normal, const"
                    +" TenseurHH & aHH, const TenseurHH & bHH);");
       if (Dabs(b3HH.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                    +"Tenseur3HHHH::Tenseur3HHHH(bool normal, const"
                    +" TenseurHH & aHH, const TenseurHH & bHH);");
    #endif
    if (normal)
     { for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          t[(ij-1)*6+kl-1] =   a3HH(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij))
                             * b3HH(cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl));
       }
    else
     { for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3HHHH.idx_i(ij); int j = cdex3HHHH.idx_j(ij);
           int k = cdex3HHHH.idx_i(kl); int l = cdex3HHHH.idx_j(kl);
           t[(ij-1)*6+kl-1] =  0.25 * (
                a3HH(i,k) * b3HH(j,l) + a3HH(j,k) * b3HH(i,l)
               +a3HH(i,l) * b3HH(j,k) + a3HH(j,l) * b3HH(i,k)
                                      );
         };
       }
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHHH::~Tenseur3HHHH() 
{ listdouble36.erase(ipointe);} ; // suppression de l'élément de la liste

// constructeur a partir d'une instance non differenciee  
// dans le cas d'un tenseur quelconque, celui-ci
// est converti à condition que les symétries existent sinon erreur en debug
// opération longue dans ce cas !
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHHH::Tenseur3HHHH ( const TenseurHHHH & B) :  
 ipointe()
  { dimension = 33; 
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 33)
//      { cout << "\n erreur de dimension, elle devrait etre = 33 ";
//        cout << "\n Tenseur3HHHH::Tenseur3HHHH ( TenseurHHHH &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 33 ) // cas d'un tenseur du même type
      { for (int i=0;i < 36;i++)
          t[i] = B.t[i];
      }
    else
      {// cas d'un tenseur quelconque
       double Z=B.MaxiComposante();
       for (int i=1;i < 4;i++)
         for (int j=1;j<=i;j++)
           for (int k=1;k < 4;k++)
             for (int l=1;l<=k;l++)
             	{// on teste les symétries et on affecte 
             	 double a = B(i,j,k,l);
              #ifdef MISE_AU_POINT 
                 if ((!diffpourcent(a,B(j,i,k,l),Z,ConstMath::unpeupetit)
                    && !diffpourcent(a,B(i,j,l,k),Z,ConstMath::unpeupetit))
                    || (Abs(Z) < ConstMath::trespetit) )
                 // erreur d'affectation
                 if (ParaGlob::NiveauImpression() > 5)
                    cout << "\n tenseurHHHH (ijkl= " << i << "," << j << "," << k << "," << l << ")= " 
                         << a << " " << B(j,i,k,l) << " " <<B(i,j,l,k) ; 
                 if (ParaGlob::NiveauImpression() > 1)
                   cout << "WARNING ** erreur constructeur, tenseur non symetrique, Tenseur3HHHH::Tenseur3HHHH(const TenseurHHHH & B)";
              #endif
                 // si il y a un pb normalement il y a eu un message
                 this->Change(i,j,k,l,a);
             	}
      };
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHHH::Tenseur3HHHH (  const Tenseur3HHHH & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i < 36;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur3HHHH::Inita(double val)   
	{ for (int i=0;i< 36;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::operator + ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 33) Message(3,"Tenseur3HHHH::operator + ( etc..");
     #endif
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur3HHHH::operator += ( const TenseurHHHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHHH::operator += ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::operator - () const 
  {  TenseurHHHH * res;
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::operator - ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHHH::operator - ( etc..");
     #endif
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3HHHH::operator -= ( const TenseurHHHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHHH::operator -= ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::operator = ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHHH::operator = ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::operator * ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3HHHH::operator *= ( const double & b)
  {for (int i = 0; i < 36; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::operator / ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3HHHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i < 36; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3HHHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3HHHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i < 36; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur3HHHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3) 
           Message(3,"Tenseur3HHHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3BB(cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3BB(cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl))
                          + a3BB(cdex3HHHH.idx_j(kl),cdex3HHHH.idx_i(kl))
                         );
       };
//     // pour vérif
//     for (int i=1;i<4;i++) for (int j=1;j<4;j++)
//      {res->Coor(i,j)=0;
//       for (int k=1;k<4;k++) for (int l=1;l<4;l++)
//         res->Coor(i,j) += (*this)(i,j,k,l) * a3BB(k,l);
//      };
   
   return *res ;
  };

    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
#ifndef MISE_AU_POINT
  inline
#endif
TenseurHH&  Tenseur3HHHH::ContractionVerticale( const TenseurBB & aBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3)
           Message(3,"Tenseur3HHHH::ContractionVerticale( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3BB(cdex3HHHH.idx_j(kl),cdex3HHHH.idx_i(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3BB(cdex3HHHH.idx_j(kl),cdex3HHHH.idx_i(kl))
                          + a3BB(cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl))
                         );
       };
//     // pour vérif
//     for (int i=1;i<4;i++) for (int j=1;j<4;j++)
//      {res->Coor(i,j)=0;
//       for (int k=1;k<4;k++) for (int l=1;l<4;l++)
//         res->Coor(i,j) += (*this)(i,j,k,l) * a3BB(k,l);
//      };
   
   return *res ;
  };



    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurHHHH& Tenseur3HHHH::operator && ( const TenseurBBHH & aBBHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 33)
           Message(3,"Tenseur3HHHH::operator && ( const TenseurBBHH & aBBHH)");
     #endif
     TenseurHHHH * res;
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BBHH & a3BBHH = *((Tenseur3BBHH*) &aBBHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3HHHH.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3BBHH.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3BBHH.t[(ef-1)*6+kl-1];
        };
     return *res;
  };

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurHHBB& Tenseur3HHHH::operator && ( const TenseurBBBB & aBBBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 33)
           Message(3,"Tenseur3HHHH::operator && ( const TenseurBBBB & aBBBB)");
     #endif
     TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BBBB & a3BBBB = *((Tenseur3BBBB*) &aBBBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3HHHH.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3BBBB.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3BBBB.t[(ef-1)*6+kl-1];
        };
     return *res;
  };

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  Tenseur3HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH) 
  { TenseurHHHH * res;
    res =  new Tenseur3HHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(ij-1)*6+kl-1] =   a3HH(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij))
                                  * b3HH(cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl));
    return *res;                         
  };      

    //fonctions définissant le produit tensoriel croisé de deux tenseurs
    // *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline
#endif
TenseurHHHH &  Tenseur3HHHH::Prod_tensoriel_croise(const TenseurHH & aHH, const TenseurHH & bHH)
  { TenseurHHHH * res;
    res =  new Tenseur3HHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel_croise(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }
       if (Dabs(b3HH.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel_croise(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3HHHH.idx_i(ij); int j = cdex3HHHH.idx_j(ij);
           int k = cdex3HHHH.idx_i(kl); int l = cdex3HHHH.idx_j(kl);
           res->t[(ij-1)*6+kl-1] =   a3HH(i,k) * b3HH(j,l);
         }
    return *res;
  };

    //fonctions définissant le produit tensoriel croisé de deux tenseurs
    // *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline
#endif
TenseurHHHH &  Tenseur3HHHH::Prod_tensoriel_croise_croise(const TenseurHH & aHH, const TenseurHH & bHH)
  { TenseurHHHH * res;
    res =  new Tenseur3HHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel_croise(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }
       if (Dabs(b3HH.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel_croise(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3HHHH.idx_i(ij); int j = cdex3HHHH.idx_j(ij);
           int k = cdex3HHHH.idx_i(kl); int l = cdex3HHHH.idx_j(kl);
           res->t[(ij-1)*6+kl-1] =   a3HH(i,l) * b3HH(j,k);
         }
    return *res;
  };

    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  Tenseur3HHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH) 
  { TenseurHHHH * res;
    res =  new Tenseur3HHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3HHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3HHHH.idx_i(ij); int j = cdex3HHHH.idx_j(ij);
           int k = cdex3HHHH.idx_i(kl); int l = cdex3HHHH.idx_j(kl);
           res->t[(ij-1)*6+kl-1] =  0.25 * (
                a3HH(i,k) * b3HH(j,l) + a3HH(j,k) * b3HH(i,l)
               +a3HH(i,l) * b3HH(j,k) + a3HH(j,l) * b3HH(i,k)
                                      );
         };
    return *res;                         
  };      

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::Transpose1et2avec3et4() const   
   { TenseurHHHH * res;
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(kl-1)*6+ij-1] = t[(ij-1)*6+kl-1] ;
     return *res;
   }; 


    // il s'agit ici de calculer la variation d'un tenseur dans une nouvelle base
    //  --> variation par rapport aux composantes covariantes d'un tenseur (ex: composantes eps_ij)

    //  d sigma^ij /  d eps_kl = d gamma^i_{.a} / d eps_kl . sigma^ab . gamma^j_{.b}
    //                          + gamma^i_{.a} . d sigma^ab / d eps_kl. gamma^j_{.b}
    //                          + gamma^i^_{.a} . sigma^ab . d gamma^j_{.b} / d eps_kl

    // connaissant sa variation dans la base actuelle
    // var_tensHHHH : en entrée: la variation du tenseur dans la base initiale qu'on appelle g_i
    //   ex: var_tensHHHH(ijkl) = d A^ij / d eps_kl
    //            : en sortie: la variation du tenseur dans la base finale qu'on appelle gp_i
    // gamma      : en entrée gpH(i) = gamma(i,j) * gH(j)
    // var_gamma   : en entrée : la variation de gamma
    //   ex: var_gamma(i,j,k,l) = d gamma^i_{.j} / d eps_kl
    // tensHH     : le tenseur dont on cherche la variation
    /// -- pour mémoire ---
        // changement de base (cf. théorie) : la matrice beta est telle que:
    // gpB(i) = beta(i,j) * gB(j) <==> gp_i = beta_i^j * g_j
    // et la matrice gamma telle que:
    // gamma(i,j) represente les coordonnees de la nouvelle base duale gpH dans l'ancienne gH
    // gpH(i) = gamma(i,j) * gH(j), i indice de ligne, j indice de colonne
    //   c-a-d= gp^i = gamma^i_j * g^j
    // rappel des différentes relations entre beta et gamma
    // [beta]^{-1} = [gamma]^T      ; [beta]^{-1T} = [gamma]
    // [beta] = [gamma]^{-1T}      ; [beta]^{T} = [gamma]^{-1}
    // changement de base pour un tenseur en  deux fois covariants:
    // [Ap^kl] = [gamma] * [A^ij] * [gamma]^T

#ifndef MISE_AU_POINT
  inline
#endif
TenseurHHHH &  Tenseur3HHHH::Var_tenseur_dans_nouvelle_base
              (const Mat_pleine& gamma,Tenseur3HHHH& var_tensHHHH, const Tableau2 <Tenseur3HH>& var_gamma
              ,const Tenseur3HH& tensHH)
 {
    TenseurHHHH * res;
    res =  new Tenseur3HHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    //  d sigma^ij /  d eps_kl = d gamma^i_{.a} / d eps_kl . sigma^ab . gamma^j_{.b}
    //                          + gamma^i_{.a} . d sigma^ab / d eps_kl. gamma^j_{.b}
    //                          + gamma^i^_{.a} . sigma^ab . d gamma^j_{.b} / d eps_kl
    for (int i=1;i<4;i++)
     for (int j=1;j<4;j++)
      for (int k=1;k<4;k++)
       for (int l=1;l<4;l++)
        { double d_sig_ij_d_eps_kl = 0.;
          for (int a=1;a<4;a++)
           for (int b=1;b<4;b++)
             d_sig_ij_d_eps_kl += var_gamma(i,a)(k,l) * tensHH(a,b) * gamma(j,b)
                                  + gamma(i,a) *  var_tensHHHH(a,b,k,l) * gamma(j,b)
                                  + gamma(i,a) * tensHH(a,b) * var_gamma(j,b)(k,l);
          res->Change(i,j,k,l,d_sig_ij_d_eps_kl);
        };

    return *res;
};

    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur3HHHH::Affectation_trans_dimension(const TenseurHHHH & aHHHH,bool plusZero)
   { switch (abs(aHHHH.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
                t[(ij-1)*6+kl-1] = aHHHH(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)
                                         ,cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl));
           break;
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
               { if ((ij < 3) && (kl < 3)) // on affecte que les grandeurs qui existent
                   t[(ij-1)*6+kl-1] = aHHHH(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)
                                         ,cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl));
               };
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHHH(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHHH.Dimension()))
                    +"n'est pas prise en compte \n Tenseur3HHHH::Affectation_trans_dimension(");
       };
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // transférer un tenseur général accessible en indice, dans un tenseur 36 Tenseur3HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
void Tenseur3HHHH::TransfertDunTenseurGeneral(const TenseurHHHH & aHHHH)
   { for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          t[(ij-1)*6+kl-1] = aHHHH(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)
                                   ,cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl));
   };
       
    // ---- manipulation d'indice ----
    // on baisse les deux premiers indices -> création d'un tenseur TenseurBBHH 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH& Tenseur3HHHH::Baisse2premiersIndices()
   { TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ijkl=0;ijkl<36;ijkl++)
          res->t[ijkl] = t[ijkl] ;
     return *res;
   }; 
    // on baisse les deux derniers indices -> création d'un tenseur TenseurHHBB 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB& Tenseur3HHHH::Baisse2derniersIndices()
   { TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ijkl=0;ijkl<36;ijkl++)
          res->t[ijkl] = t[ijkl] ;
     return *res;
   }; 
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::Baselocale(TenseurHHHH & A,const BaseH & gi) const
   {  return produit44(A,*this,gi);
   };
    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat qui a la même dimension
    // retour d'une reference sur A
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & Tenseur3HHHH::ChangeBase(TenseurHHHH & A,const BaseB & gi) const
   {  return produit55(A,*this,gi);
   };
             
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur3HHHH::operator == ( const TenseurHHHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHHH::operator == ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur3HHHH::Change (int i, int j, int k, int l, const double& val) 
  { t[(cdex3HHHH.odVect(i,j)-1)*6+cdex3HHHH.odVect(k,l)-1]  = val;}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur3HHHH::ChangePlus (int i, int j, int k, int l, const double& val)
  { t[(cdex3HHHH.odVect(i,j)-1)*6+cdex3HHHH.odVect(k,l)-1]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur3HHHH::operator () (int i, int j, int k, int l) const 
 { return t[(cdex3HHHH.odVect(i,j)-1)*6+cdex3HHHH.odVect(k,l)-1]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur3HHHH::MaxiComposante() const
  { return DabsMaxiTab(t,  36) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur3HHHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur3HHHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur3HHHH ";
    // puis les datas
     for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };

#ifndef MISE_AU_POINT
  inline
#endif
// affichage sous forme de tableau bidim
void Tenseur3HHHH::Affiche_bidim(ostream & sort) const
 {
    sort << "\n" ;
    for (int kl=1;kl < 7;kl++)
      sort << setw(15) << kl ;
    for (int ij=1;ij < 7;ij++)
      {sort << '\n'<< setw(4) << ij;
       for (int kl=1;kl < 7;kl++)
             { int i= cdex3HHHH.idx_i(ij); int j= cdex3HHHH.idx_j(ij);
               int k= cdex3HHHH.idx_i(kl); int l= cdex3HHHH.idx_j(kl);
               sort //<<" ("<<i<<","<<j<<","<<k<<","<<l<<")"
                    << setw(15) << setprecision(7)
                    << t[(cdex3HHHH.odVect(i,j)-1)*6+cdex3HHHH.odVect(k,l)-1];
              
             }
       sort << '\n';
      }
    cout << endl ;
 };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur3HHHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 33) A.Message(3,"operator >> (istream & entree, Tenseur3HHHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3HHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur3HHHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur3HHHH ";
    // puis les datas
    for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur3HHHH::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3) 
           Message(3,"TenseurQ3geneHHHH::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
   
     for (int kl=1;kl < 7;kl++)
      {for (int ij=1;ij < 4;ij++)
         // partie simple produit : la partie diagonale
         res->Coor(cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl)) +=
             a3BB(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij)) * t[(ij-1)*6+kl-1];
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int ij=4;ij < 7;ij++)
         res->Coor(cdex3HHHH.idx_i(kl),cdex3HHHH.idx_j(kl)) +=
                         (  a3BB(cdex3HHHH.idx_i(ij),cdex3HHHH.idx_j(ij))
                          + a3BB(cdex3HHHH.idx_j(ij),cdex3HHHH.idx_i(ij))
                         )
                       *  t[(ij-1)*6+kl-1] ;
       };
   
//     // pour vérif
//     for (int i=1;i<4;i++) for (int j=1;j<4;j++)
//      {res->Coor(i,j)=0;
//       for (int k=1;k<4;k++) for (int l=1;l<4;l++)
//         res->Coor(i,j) += a3BB(i,j) * (*this)(i,j,k,l) ;
//      };

     return *res ;
  };
//=========== fin fonction protected ======================	
 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBBB::ChangementIndex::ChangementIndex() :
  idx_i(6),idx_j(6),odVect(3)
  { idx_i(1)=1;idx_i(2)=2;idx_i(3)=3;idx_i(4)=2;idx_i(5)=3;idx_i(6)=3;
    idx_j(1)=1;idx_j(2)=2;idx_j(3)=3;idx_j(4)=1;idx_j(5)=2;idx_j(6)=1;
    odVect(1,1)=1;odVect(1,2)=4;odVect(1,3)=6;
    odVect(2,1)=4;odVect(2,2)=2;odVect(2,3)=5;
    odVect(3,1)=6;odVect(3,2)=5;odVect(3,3)=3;
  }; 
// variables globales
//Tenseur3BBBB::ChangementIndex  Tenseur3BBBB::cdex3BBBB;


    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBBB::Tenseur3BBBB() :  
 ipointe() // par défaut
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBBB::Tenseur3BBBB( const double val)  :  
 ipointe()
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    // booleen = false : produit tensoriel barre
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBBB::Tenseur3BBBB(bool normal, const TenseurBB & aBB, const TenseurBB & bBB) :  
 ipointe()
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                   +"Tenseur3BBBB::Tenseur3BBBB(bool normal, const"
                    +" TenseurBB & aBB, const TenseurBB & bBB);");
       if (Dabs(b3BB.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                   +"Tenseur3BBBB::Tenseur3BBBB(bool normal, const"
                    +" TenseurBB & aBB, const TenseurBB & bBB);");
    #endif
    if (normal)
     { for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          t[(ij-1)*6+kl-1] =   a3BB(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij))
                             * b3BB(cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl));
       }
    else
     { for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3BBBB.idx_i(ij); int j = cdex3BBBB.idx_j(ij);
           int k = cdex3BBBB.idx_i(kl); int l = cdex3BBBB.idx_j(kl);
           t[(ij-1)*6+kl-1] =  0.25 * (
                a3BB(i,k) * b3BB(j,l) + a3BB(j,k) * b3BB(i,l)
               +a3BB(i,l) * b3BB(j,k) + a3BB(j,l) * b3BB(i,k)
                                      );
         };
       }
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBBB::~Tenseur3BBBB() 
{ listdouble36.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBBB::Tenseur3BBBB ( const TenseurBBBB & B) :  
 ipointe()
  { dimension = 33;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 33)
//      { cout << "\n erreur de dimension, elle devrait etre = 33 ";
//        cout << "\n Tenseur3BBBB::Tenseur3BBBB ( TenseurBBBB &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 33 ) // cas d'un tenseur du même type
      { for (int i=0;i < 36;i++)
          t[i] = B.t[i];
      }
    else
      {// cas d'un tenseur quelconque
       double Z=B.MaxiComposante();
       for (int i=1;i < 4;i++)
         for (int j=1;j<=i;j++)
           for (int k=1;k < 4;k++)
             for (int l=1;l<=k;l++)
             	{// on teste les symétries et on affecte 
             	 double a = B(i,j,k,l);
              #ifdef MISE_AU_POINT 
                 if ((!diffpourcent(a,B(j,i,k,l),Z,ConstMath::unpeupetit)
                    && !diffpourcent(a,B(i,j,l,k),Z,ConstMath::unpeupetit))
                    || (Abs(Z) < ConstMath::trespetit) )
                 // erreur d'affectation
                 if (ParaGlob::NiveauImpression() > 5)
                    cout << "\n tenseurBBBB (ijkl= " << i << "," << j << "," << k << "," << l << ")= " 
                         << a << " " << B(j,i,k,l) << " " <<B(i,j,l,k) ; 
                 if (ParaGlob::NiveauImpression() > 1)
                   cout << "WARNING ** erreur constructeur, tenseur non symetrique, Tenseur3BBBB::Tenseur3BBBB(const TenseurBBBB & B)";
              #endif
                 // si il y a un pb normalement il y a eu un message
                 this->Change(i,j,k,l,a);
             	}
      };
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBBB::Tenseur3BBBB (  const Tenseur3BBBB & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i < 36;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur3BBBB::Inita(double val)   
	{ for (int i=0;i< 36;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::operator + ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 33) Message(3,"Tenseur3BBBB::operator + ( etc..");
     #endif
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur3BBBB::operator += ( const TenseurBBBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBBB::operator += ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::operator - () const 
  {  TenseurBBBB * res;
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::operator - ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBBB::operator - ( etc..");
     #endif
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3BBBB::operator -= ( const TenseurBBBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBBB::operator -= ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::operator = ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBBB::operator = ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::operator * ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i < 36; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3BBBB::operator *= ( const double & b)
  {for (int i = 0; i < 36; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::operator / ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3BBBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i < 36; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3BBBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3BBBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i < 36; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur3BBBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3) 
           Message(3,"Tenseur3BBBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3HH(cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3HH(cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl))
                          + a3HH(cdex3BBBB.idx_j(kl),cdex3BBBB.idx_i(kl))
                         );
       };
   return *res ;
  };

    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
#ifndef MISE_AU_POINT
  inline
#endif
TenseurBB&  Tenseur3BBBB::ContractionVerticale( const TenseurHH & aHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3)
           Message(3,"Tenseur3BBBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3HH(cdex3BBBB.idx_j(kl),cdex3BBBB.idx_i(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3HH(cdex3BBBB.idx_j(kl),cdex3BBBB.idx_i(kl))
                          + a3HH(cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl))
                         );
       };
   return *res ;
  };
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurBBBB& Tenseur3BBBB::operator && ( const TenseurHHBB & aHHBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 33)
           Message(3,"Tenseur3BBBB::operator && ( const TenseurHHBB & aHHBB)");
     #endif
     TenseurBBBB * res;
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HHBB & a3HHBB = *((Tenseur3HHBB*) &aHHBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3BBBB.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3HHBB.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3HHBB.t[(ef-1)*6+kl-1];
        };
     return *res;
  };

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurBBHH& Tenseur3BBBB::operator && ( const TenseurHHHH & aHHHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 33)
           Message(3,"Tenseur3BBBB::operator && ( const TenseurHHHH & aHHHH)");
     #endif
     TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HHHH & a3HHHH = *((Tenseur3HHHH*) &aHHHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3BBBB.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3HHHH.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3HHHH.t[(ef-1)*6+kl-1];
        };
     return *res;
  };

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  Tenseur3BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB) 
  { TenseurBBBB * res;
    res =  new Tenseur3BBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(ij-1)*6+kl-1] =   a3BB(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij))
                                  * b3BB(cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl));
    return *res;                         
  };      
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  Tenseur3BBBB::Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurBB & bBB) 
  { TenseurBBBB * res;
    res =  new Tenseur3BBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3BBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3BBBB.idx_i(ij); int j = cdex3BBBB.idx_j(ij);
           int k = cdex3BBBB.idx_i(kl); int l = cdex3BBBB.idx_j(kl);
           res->t[(ij-1)*6+kl-1] =  0.25 * (
                a3BB(i,k) * b3BB(j,l) + a3BB(j,k) * b3BB(i,l)
               +a3BB(i,l) * b3BB(j,k) + a3BB(j,l) * b3BB(i,k)
                                      );
         };
    return *res;                         
  };      


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::Transpose1et2avec3et4() const   
   { TenseurBBBB * res;
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(kl-1)*6+ij-1] = t[(ij-1)*6+kl-1] ;
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur3BBBB::Affectation_trans_dimension(const TenseurBBBB & aBBBB,bool plusZero)
   { switch (abs(aBBBB.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
                t[(ij-1)*6+kl-1] = aBBBB(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)
                                         ,cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl));
           break;
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
               { if ((ij < 3) && (kl < 3)) // on affecte que les grandeurs qui existent
                   t[(ij-1)*6+kl-1] = aBBBB(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)
                                         ,cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl));
               };
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBBB(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBBB.Dimension()))
                    +"n'est pas prise en compte \n Tenseur3BBBB::Affectation_trans_dimension(");
       };
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // transférer un tenseur général de même dimension accessible en indice, dans un tenseur 36 Tenseur3HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
void Tenseur3BBBB::TransfertDunTenseurGeneral(const TenseurBBBB & aBBBB)
   { for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          t[(ij-1)*6+kl-1] = aBBBB(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)
                                   ,cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl));
   };
       
    // ---- manipulation d'indice ----
    // on monte les deux premiers indices -> création d'un tenseur TenseurHHBB 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB& Tenseur3BBBB::Monte2premiersIndices()
   { TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ijkl=0;ijkl<36;ijkl++)
          res->t[ijkl] = t[ijkl] ;
     return *res;
   }; 
    // on monte les deux derniers indices -> création d'un tenseur TenseurBBHH 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH& Tenseur3BBBB::Monte2derniersIndices()
   { TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ijkl=0;ijkl<36;ijkl++)
          res->t[ijkl] = t[ijkl] ;
     return *res;
   };
   
// on monte les 4 indices -> création d'un tenseur TenseurHHHH 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH& Tenseur3BBBB::Monte4Indices()    
   { TenseurHHHH * res;
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ijkl=0;ijkl<36;ijkl++)
          res->t[ijkl] = t[ijkl] ;
     return *res;
   };
              
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::Baselocale(TenseurBBBB & A,const BaseB & gi) const
   {  return produit44(A,*this,gi);
   };
    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat qui a la même dimension
    // retour d'une reference sur A
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & Tenseur3BBBB::ChangeBase(TenseurBBBB & A,const BaseH & gi) const
   {  return produit55(A,*this,gi);
   };
             
// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur3BBBB::operator == ( const TenseurBBBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBBB::operator == ( etc..");
     #endif
     for (int i = 0; i < 36; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur3BBBB::Change (int i, int j, int k, int l,const double& val) 
  { t[(cdex3BBBB.odVect(i,j)-1)*6+cdex3BBBB.odVect(k,l)-1]  = val;}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur3BBBB::ChangePlus (int i, int j, int k, int l,const double& val)
  { t[(cdex3BBBB.odVect(i,j)-1)*6+cdex3BBBB.odVect(k,l)-1]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur3BBBB::operator () (int i, int j, int k, int l) const 
 { return t[(cdex3BBBB.odVect(i,j)-1)*6+cdex3BBBB.odVect(k,l)-1]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur3BBBB::MaxiComposante() const
  { return DabsMaxiTab(t,  36) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur3BBBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur3BBBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur3BBBB ";
    // puis les datas
     for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };

#ifndef MISE_AU_POINT
  inline
#endif
// affichage sous forme de tableau bidim
void Tenseur3BBBB::Affiche_bidim(ostream & sort) const
 {
    sort << "\n" ;
    for (int kl=1;kl < 7;kl++)
      sort << setw(15) << kl ;
    for (int ij=1;ij < 7;ij++)
      {sort << '\n'<< setw(4) << ij;
       for (int kl=1;kl < 7;kl++)
             { int i= cdex3BBBB.idx_i(ij); int j= cdex3BBBB.idx_j(ij);
               int k= cdex3BBBB.idx_i(kl); int l= cdex3BBBB.idx_j(kl);
               sort << setw(15) << setprecision(7)
                    << t[(cdex3BBBB.odVect(i,j)-1)*6+cdex3BBBB.odVect(k,l)-1];
             }
       sort << '\n';
      }
    cout << endl ;
 };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur3BBBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 33) A.Message(3,"operator >> (istream & entree, Tenseur3BBBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3BBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur3BBBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur3BBBB ";
    // puis les datas
    for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur3BBBB::Prod_gauche( const TenseurHH & aHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3) 
           Message(3,"Tenseur3BBBB::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
   
     for (int kl=1;kl < 7;kl++)
      {for (int ij=1;ij < 4;ij++)
         res->Coor(cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl)) +=
              a3HH(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij)) * t[(ij-1)*6+kl-1];
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int ij=4;ij < 7;ij++)
         res->Coor(cdex3BBBB.idx_i(kl),cdex3BBBB.idx_j(kl)) +=
                         (  a3HH(cdex3BBBB.idx_i(ij),cdex3BBBB.idx_j(ij))
                          + a3HH(cdex3BBBB.idx_j(ij),cdex3BBBB.idx_i(ij))
                         )
                       *  t[(ij-1)*6+kl-1] ;
       };
   
//     // pour vérif
//     for (int i=1;i<4;i++) for (int j=1;j<4;j++)
//      {res->Coor(i,j)=0;
//       for (int k=1;k<4;k++) for (int l=1;l<4;l++)
//         res->Coor(i,j) += a3HH(i,j) * (*this)(i,j,k,l) ;
//      };

     return *res ;
  };
//=========== fin fonction protected ======================	

  
  
#endif
