

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ3gene.h"
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur3.h"
#include "CharUtil.h"

#ifndef  TenseurQ3gene_H_deja_inclus



///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
//?????????????????????????????????,a faire les divers cas de produit tensoriel de tenseur du quatrieme ordre
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,
///?????????????????????????????????,


// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes 3HHHH
//------------------------------------------------------------------   

// Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneHHHH::TenseurQ3geneHHHH() :  
 ipointe() // par défaut
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<81;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneHHHH::TenseurQ3geneHHHH( const double val) :  
 ipointe() 
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<81;i++) t[i]=val;
  };
  
    // cas = 1      : produit tensoriel normal 
    //                  *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    // cas = 2      : produit tensoriel barre
    //                  *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
    // cas = 3      : produit tensoriel under barre
    //                  *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneHHHH::TenseurQ3geneHHHH(int cas, const TenseurHH & aHH, const TenseurHH & bHH) :  
 ipointe() 
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          Message(3,strcat(strcat("produit tensoriel a partir d'un premier tenseur non symétriques \n",
                   "TenseurQ3geneHHHH::TenseurQ3geneHHHH(bool normal, const"),
                    " TenseurHH & aHH, const TenseurHH & bHH);"));
       if (Dabs(b3HH.Dimension()) != 3) 
          Message(3,strcat(strcat("produit tensoriel a partir d'un second tenseur non symétriques \n",
                   "TenseurQ3geneHHHH::TenseurQ3geneHHHH(bool normal, const"),
                    " TenseurHH & aHH, const TenseurHH & bHH);"));
    #endif
    switch (cas)
     { case 1 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
               // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3HH(i,j) * b3HH(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3HH(i,k) * b3HH(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3HH(i,l) * b3HH(j,k);
           break;   
         }
       default :
        { Message(3,"TenseurQ3geneHHHH::TenseurQ3geneHHHH(int cas, const TenseurHH & aHH, const TenseurHH & bHH)");
          Sortie(1);
         }
      }    
  }; 
 
// idem pour des instances de tenseurs 3       
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneHHHH::TenseurQ3geneHHHH(int cas, const Tenseur3HH & a3HH, const Tenseur3HH & b3HH) :  
 ipointe()
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    switch (cas)
     { case 1 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
               // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3HH(i,j) * b3HH(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3HH(i,k) * b3HH(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3HH(i,l) * b3HH(j,k);
           break;   
         }
       default :
        { Message(3,"TenseurQ3geneHHHH::TenseurQ3geneHHHH(int cas, const Tenseur3HH & a3HH, const Tenseur3HH & b3HH)");
          Sortie(1);
         }
      }    
  };      

    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneHHHH::~TenseurQ3geneHHHH() 
{ listdouble81.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneHHHH::TenseurQ3geneHHHH ( const TenseurHHHH & B) :  
 ipointe()
  { dimension = B.dimension;
    #ifdef MISE_AU_POINT
    if (Dabs(dimension) != 30)
      { cout << "\n erreur de dimension, elle devrait etre = 30 ";
        cout << "\n TenseurQ3geneHHHH::TenseurQ3geneHHHH ( TenseurHHHH &) " << endl;
        Sortie(1);
      }  
    #endif
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 81;i++)
       t[i] = B.t[i];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneHHHH::TenseurQ3geneHHHH (  const TenseurQ3geneHHHH & B):  
 ipointe()
  { this->dimension = B.dimension;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 81;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ3geneHHHH::Inita(double val)   
	{ for (int i=0;i< 81;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3geneHHHH::operator + ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 30) Message(3,"TenseurQ3geneHHHH::operator + ( etc..");
     #endif
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ3geneHHHH::operator += ( const TenseurHHHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneHHHH::operator += ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3geneHHHH::operator - () const 
  {  TenseurHHHH * res;
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3geneHHHH::operator - ( const TenseurHHHH & B) const 
  {  TenseurHHHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneHHHH::operator - ( etc..");
     #endif
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3geneHHHH::operator -= ( const TenseurHHHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneHHHH::operator -= ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3geneHHHH::operator = ( const TenseurHHHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneHHHH::operator = ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3geneHHHH::operator * ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3geneHHHH::operator *= ( const double & b)
  {for (int i = 0; i< 81; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3geneHHHH::operator / ( const double & b) const 
  {  TenseurHHHH * res;
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3geneHHHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 81; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3geneHHHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3geneHHHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 81; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& TenseurQ3geneHHHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3) 
           Message(3,"TenseurQ3geneHHHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<=3;i++) for (int j=1;j<=i;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       // (((i-1)3+(j-1))3+(k-1))3+l-1
         res->Coor(i,j) += t[27*i+9*j+3*k+l-40] * a3BB(k,l);
     return *res ;
  };

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurHHBB& TenseurQ3geneHHHH::operator && ( const TenseurBBBB & aBBBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 30) 
           Message(3,"TenseurQ3geneHHHH::Prod_gauche( const TenseurBBBB & F)");
     #endif
     TenseurHHBB * res;
     res =  new TenseurQ3geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneBBBB & a3BBBB = *((TenseurQ3geneBBBB*) &aBBBB); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3BBBB.t[27*l+9*k+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };

#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurHHHH& TenseurQ3geneHHHH::operator && ( const TenseurBBHH & aBBHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 30) 
           Message(3,"TenseurQ3geneHHHH::Prod_gauche( const TenseurBBHH & F)");
     #endif
     TenseurHHHH * res;
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneBBHH & a3BBHH = *((TenseurQ3geneBBHH*) &aBBHH); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3BBHH.t[27*l+9*k+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche 
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre deux
TenseurHH& TenseurQ3geneHHHH::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3) 
           Message(3,"TenseurQ3geneHHHH::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<=3;i++) for (int j=1;j<=i;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       // (((i-1)3+(j-1))3+(k-1))3+l-1
         res->Coor(i,j) +=  a3BB(k,l) * t[27*k+9*l+3*i+j-40] ;
     return *res ;
  };
  
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurBBHH& TenseurQ3geneHHHH::Prod_gauche( const TenseurBBBB & aBBBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 30) 
           Message(3,"TenseurQ3geneHHHH::Prod_gauche( const TenseurBBBB & F)");
     #endif
     TenseurBBHH * res;
     res =  new TenseurQ3geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneBBBB & a3BBBB = *((TenseurQ3geneBBBB*) &aBBBB); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3BBBB.t[27*i+9*j+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHBB
TenseurHHHH& TenseurQ3geneHHHH::Prod_gauche( const TenseurHHBB & aHHBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 30) 
           Message(3,"TenseurQ3geneHHHH::Prod_gauche( const TenseurHHBB & F)");
     #endif
     TenseurHHHH * res;
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneHHBB & a3HHBB = *((TenseurQ3geneHHBB*) &aHHBB); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3HHBB.t[27*l+9*k+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };
//=========== fin fonction protected ======================	

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  TenseurQ3geneHHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH) 
  { TenseurHHHH * res;
    res =  new TenseurQ3geneHHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ3geneHHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ3geneHHHH::Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] = a3HH(i,j) * b3HH(k,l);
    return *res;                         
  };      
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH &  TenseurQ3geneHHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH) 
  { TenseurHHHH * res;
    res =  new TenseurQ3geneHHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ3geneHHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ3geneHHHH::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] = a3HH(i,k) * b3HH(j,l);
    return *res;                         
  };      

#ifndef MISE_AU_POINT
  inline 
#endif
    // *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
TenseurHHHH &  TenseurQ3geneHHHH::Prod_tensoriel_under_barre(const TenseurHH & aHH, const TenseurHH & bHH) 
  { TenseurHHHH * res;
    res =  new TenseurQ3geneHHHH;
    LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ3geneHHHH::Prod_tensoriel_under_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ3geneHHHH::Prod_tensoriel_under_barre(const TenseurHH & aHH, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] = a3HH(i,l) * b3HH(j,k);
    return *res;                         
  };      

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHHH & TenseurQ3geneHHHH::Transpose1et2avec3et4() const   
   { TenseurHHHH * res;
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       // (((i-1)3+(j-1))3+(k-1))3+l-1
       res->t[27*k+9*l+3*i+j-40] = t[27*i+9*j+3*k+l-40];
     return *res;
   };           
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ3geneHHHH::Affectation_trans_dimension(const TenseurHHHH & aHHHH,bool plusZero)
   { switch (abs(aHHHH.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int i=1;i<4;i++)
            for (int j=1;j<4;j++)
             for (int k=1;k<4;k++)
              for (int l=1;l<4;k++)
                t[27*k+9*l+3*i+j-40] = aHHHH(i,j,k,l);
           break;
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[27*k+9*l+3*i+j-40] = aHHHH(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHHH(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHHH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ3geneHHHH::Affectation_trans_dimension(");
       };
   };

#ifndef MISE_AU_POINT
  inline 
#endif
    // création d'un tenseur symétrique / au deux premiers indices et / au deux derniers indices
    // B(i,i,i,i) = A(i,i,i,i); B(i,j,k,k) = 1/2(A(i,j,k,k)+ A(j,i,k,k)); si 2 premiers indices différents
    // B(i,i,k,l) = 1/2(A(i,i,k,l)+ A(j,i,l,k)); si 2 derniers indices différents 
    // B(i,j,k,l) = 1/4(A(i,j,k,l)+ A(j,i,k,l) + A(i,j,l,k)+ A(j,i,l,k)); si tous les indices différents
TenseurHHHH & TenseurQ3geneHHHH::Symetrise1et2_3et4() const
	{TenseurHHHH * res;
     res =  new TenseurQ3geneHHHH;
     LesMaillonsHHHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // en fait si l'on veut  faire avec les tests il y a plus de travail en test qu'en calcul car le
     // le troisième cas est le plus courant donc on ne met pas de test et on divise par 4 tout le temps
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       res->t[27*i+9*j+3*k+l-40] = 0.25 * (t[27*i+9*j+3*k+l-40] + t[27*j+9*i+3*k+l-40] 
                                           + t[27*i+9*j+3*l+k-40] + t[27*j+9*i+3*l+k-40]);
     return *res;
	};

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ3geneHHHH::operator == ( const TenseurHHHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneHHHH::operator == ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
     return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ3geneHHHH::Change (int i, int j, int k, int l, const double& val) 
  { t[27*i+9*j+3*k+l-40]  = val;}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ3geneHHHH::ChangePlus (int i, int j, int k, int l, const double& val)
  { t[27*i+9*j+3*k+l-40]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ3geneHHHH::operator () (int i, int j, int k, int l) const 
 { return t[27*i+9*j+3*k+l-40]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ3geneHHHH::MaxiComposante() const
  { return DabsMaxiTab(t,81) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ3geneHHHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3geneHHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 81; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ3geneHHHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ3geneHHHH ";
    // puis les datas
     for (int i = 0; i< 81; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ3geneHHHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 30) A.Message(3,"operator >> (istream & entree, TenseurQ3geneHHHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3geneHHHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 81; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ3geneHHHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ3geneHHHH ";
    // puis les datas
    for (int i = 0; i< 81; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------


    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneBBBB::TenseurQ3geneBBBB() :  
 ipointe() // par défaut
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<81;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneBBBB::TenseurQ3geneBBBB( const double val)  :  
 ipointe()
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<81;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel avec 3 cas
    // cas = 1      : produit tensoriel normal 
    //                  *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    // cas = 2      : produit tensoriel barre
    //                  *this=aBB(i,k).bBB(j,l) gHi gHj gHk gHl
    // cas = 3      : produit tensoriel under barre
    //                  *this=aBB(i,l).bBB(j,k) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneBBBB::TenseurQ3geneBBBB(int cas, const TenseurBB & aBB, const TenseurBB & bBB) :  
 ipointe()
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          Message(3,strcat(strcat("produit tensoriel a partir d'un premier tenseur non symétriques \n",
                   "TenseurQ3geneBBBB::TenseurQ3geneBBBB(bool normal, const"),
                    " TenseurBB & aBB, const TenseurBB & bBB);"));
       if (Dabs(b3BB.Dimension()) != 3) 
          Message(3,strcat(strcat("produit tensoriel a partir d'un second tenseur non symétriques \n",
                   "TenseurQ3geneBBBB::TenseurQ3geneBBBB(bool normal, const"),
                    " TenseurBB & aBB, const TenseurBB & bBB);"));
    #endif
    switch (cas)
     { case 1 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
               // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3BB(i,j) * b3BB(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3BB(i,k) * b3BB(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3BB(i,l) * b3BB(j,k);
           break;   
         }
       default :
        { Message(3,"TenseurQ3geneBBBB::TenseurQ3geneBBBB(int cas, const TenseurBB & aBB, const TenseurBB & bBB)");
          Sortie(1);
         }
      }    
  };
  
// idem mais avec des instances de tenseur 3        
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneBBBB::TenseurQ3geneBBBB(int cas, const Tenseur3BB & a3BB, const Tenseur3BB & b3BB) :  
 ipointe()
  { dimension = 30;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    switch (cas)
     { case 1 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
               // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3BB(i,j) * b3BB(k,l);
           break;   
         }
       case 2 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3BB(i,k) * b3BB(j,l);
           break;   
         }
       case 3 :
        { for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
              // (((i-1)3+(j-1))3+(k-1))3+l-1
              t[27*i+9*j+3*k+l-40] = a3BB(i,l) * b3BB(j,k);
           break;   
         }
       default :
        { Message(3,"TenseurQ3geneBBBB::TenseurQ3geneBBBB(int cas, const Tenseur3BB & a3BB, const Tenseur3BB & b3BB)");
          Sortie(1);
         }
      }    
  };      

    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneBBBB::~TenseurQ3geneBBBB() 
{ listdouble81.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneBBBB::TenseurQ3geneBBBB ( const TenseurBBBB & B) :  
 ipointe()
  { dimension = B.dimension;
    #ifdef MISE_AU_POINT
    if (Dabs(dimension) != 30)
      { cout << "\n erreur de dimension, elle devrait etre = 30 ";
        cout << "\n TenseurQ3geneBBBB::TenseurQ3geneBBBB ( TenseurBBBB &) " << endl;
        Sortie(1);
      }  
    #endif
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 81;i++)
       t[i] = B.t[i];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ3geneBBBB::TenseurQ3geneBBBB (  const TenseurQ3geneBBBB & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble81.push_front(Reels81());  // allocation
    ipointe = listdouble81.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 81;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ3geneBBBB::Inita(double val)   
	{ for (int i=0;i< 81;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3geneBBBB::operator + ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 30) Message(3,"TenseurQ3geneBBBB::operator + ( etc..");
     #endif
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ3geneBBBB::operator += ( const TenseurBBBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneBBBB::operator += ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3geneBBBB::operator - () const 
  {  TenseurBBBB * res;
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3geneBBBB::operator - ( const TenseurBBBB & B) const 
  {  TenseurBBBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneBBBB::operator - ( etc..");
     #endif
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3geneBBBB::operator -= ( const TenseurBBBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneBBBB::operator -= ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3geneBBBB::operator = ( const TenseurBBBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneBBBB::operator = ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3geneBBBB::operator * ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 81; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3geneBBBB::operator *= ( const double & b)
  {for (int i = 0; i< 81; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3geneBBBB::operator / ( const double & b) const 
  {  TenseurBBBB * res;
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3geneBBBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 81; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ3geneBBBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ3geneBBBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 81; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ3geneBBBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3) 
           Message(3,"TenseurQ3geneBBBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<=3;i++) for (int j=1;j<=i;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       // (((i-1)3+(j-1))3+(k-1))3+l-1
         res->Coor(i,j) += t[27*i+9*j+3*k+l-40] * a3HH(k,l);
     return *res ;
  };
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHBB
TenseurBBBB& TenseurQ3geneBBBB::operator && ( const TenseurHHBB & aHHBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 30) 
           Message(3,"TenseurQ3geneBBBB::Prod_gauche( const TenseurHHBB & F)");
     #endif
     TenseurBBBB * res;
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneHHBB & a3HHBB = *((TenseurQ3geneHHBB*) &aHHBB); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3HHBB.t[27*l+9*k+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurBBHH& TenseurQ3geneBBBB::operator && ( const TenseurHHHH & aHHHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 30) 
           Message(3,"TenseurQ3geneBBBB::Prod_gauche( const TenseurHHHH & F)");
     #endif
     TenseurBBHH * res;
     res =  new TenseurQ3geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneHHHH & a3HHHH = *((TenseurQ3geneHHHH*) &aHHHH); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3HHHH.t[27*l+9*k+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ3geneBBBB::Prod_gauche( const TenseurHH & aHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3) 
           Message(3,"TenseurQ3geneBBBB::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<=3;i++) for (int j=1;j<=i;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       // (((i-1)3+(j-1))3+(k-1))3+l-1
         res->Coor(i,j) +=  a3HH(k,l) * t[27*k+9*l+3*i+j-40] ;
     return *res ;
  };
  
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurBBBB& TenseurQ3geneBBBB::Prod_gauche( const TenseurBBHH & aBBHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 30) 
           Message(3,"TenseurQ3geneBBBB::Prod_gauche( const TenseurBBHH & F)");
     #endif
     TenseurBBBB * res;
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneBBHH & a3BBHH = *((TenseurQ3geneBBHH*) &aBBHH); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3BBHH.t[27*l+9*k+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurHHBB& TenseurQ3geneBBBB::Prod_gauche( const TenseurHHHH & aHHHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 30) 
           Message(3,"TenseurQ3geneBBBB::Prod_gauche( const TenseurHHHH & F)");
     #endif
     TenseurHHBB * res;
     res =  new TenseurQ3geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ3geneHHHH & a3HHHH = *((TenseurQ3geneHHHH*) &aHHHH); // passage en dim 3
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       for(int e=1;e<=3;e++) for (int f=1;f<=3;f++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] += a3HHHH.t[27*l+9*k+3*e+f-40] * this->t[27*f+9*e+3*k+l-40];
     return *res;                         
  };
//=========== fin fonction protected ======================	
   
    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  TenseurQ3geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB) 
  { TenseurBBBB * res;
    res =  new TenseurQ3geneBBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ3geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ3geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] = a3BB(i,j) * b3BB(k,l);
    return *res;                         
  };      
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // *this=aBB(i,k).bBB(j,l) gHi gHj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB &  TenseurQ3geneBBBB::Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurBB & bBB) 
  { TenseurBBBB * res;
    res =  new TenseurQ3geneBBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ3geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ3geneBBBB::Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] = a3BB(i,k) * b3BB(j,l);
    return *res;                         
  };      



#ifndef MISE_AU_POINT
  inline 
#endif
    // *this=aBB(i,l).bBB(j,k) gHi gHj gHk gHl
TenseurBBBB &  TenseurQ3geneBBBB::Prod_tensoriel_under_barre(const TenseurBB & aBB, const TenseurBB & bBB) 
  { TenseurBBBB * res;
    res =  new TenseurQ3geneBBBB;
    LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ3geneBBBB::Prod_tensoriel_under_barre(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ3geneBBBB::Prod_tensoriel_under_barre(const TenseurBB & aBB, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
         // (((i-1)3+(j-1))3+(k-1))3+l-1
          res->t[27*i+9*j+3*k+l-40] = a3BB(i,l) * b3BB(j,k);
    return *res;                         
  };      




    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBBB & TenseurQ3geneBBBB::Transpose1et2avec3et4() const   
   { TenseurBBBB * res;
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       // (((i-1)3+(j-1))3+(k-1))3+l-1
       res->t[27*k+9*l+3*i+j-40] = t[27*i+9*j+3*k+l-40];
     return *res;
   };           
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ3geneBBBB::Affectation_trans_dimension(const TenseurBBBB & aBBBB,bool plusZero)
   { switch (abs(aBBBB.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int i=1;i<4;i++)
            for (int j=1;j<4;j++)
             for (int k=1;k<4;k++)
              for (int l=1;l<4;k++)
                t[27*k+9*l+3*i+j-40] = aBBBB(i,j,k,l);
           break;
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[27*k+9*l+3*i+j-40] = aBBBB(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBBB(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBBB.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ3geneBBBB::Affectation_trans_dimension(");
       };
   };


#ifndef MISE_AU_POINT
  inline 
#endif
    // création d'un tenseur symétrique / au deux premiers indices et / au deux derniers indices
    // B(i,i,i,i) = A(i,i,i,i); B(i,j,k,k) = 1/2(A(i,j,k,k)+ A(j,i,k,k)); si 2 premiers indices différents
    // B(i,i,k,l) = 1/2(A(i,i,k,l)+ A(j,i,l,k)); si 2 derniers indices différents 
    // B(i,j,k,l) = 1/4(A(i,j,k,l)+ A(j,i,k,l) + A(i,j,l,k)+ A(j,i,l,k)); si tous les indices différents
TenseurBBBB & TenseurQ3geneBBBB::Symetrise1et2_3et4() const
	{TenseurBBBB * res;
     res =  new TenseurQ3geneBBBB;
     LesMaillonsBBBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     // en fait si l'on veut  faire avec les tests il y a plus de travail en test qu'en calcul car le
     // le troisième cas est le plus courant donc on ne met pas de test et on divise par 4 tout le temps
     for (int i=1;i<=3;i++) for (int j=1;j<=3;j++) for (int k=1;k<=3;k++) for (int l=1;l<=3;l++)
       res->t[27*i+9*j+3*k+l-40] = 0.25 * (t[27*i+9*j+3*k+l-40] + t[27*j+9*i+3*k+l-40] 
                                           + t[27*i+9*j+3*l+k-40] + t[27*j+9*i+3*l+k-40]);
     return *res;
	};

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ3geneBBBB::operator == ( const TenseurBBBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 30) Message(3,"TenseurQ3geneBBBB::operator == ( etc..");
     #endif
     for (int i = 0; i< 81; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ3geneBBBB::Change (int i, int j, int k, int l,const double& val) 
  { t[27*i+9*j+3*k+l-40]  = val;}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ3geneBBBB::ChangePlus (int i, int j, int k, int l,const double& val)
  { t[27*i+9*j+3*k+l-40]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ3geneBBBB::operator () (int i, int j, int k, int l) const 
 { return t[27*i+9*j+3*k+l-40]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ3geneBBBB::MaxiComposante() const
  { return DabsMaxiTab(t,  81) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ3geneBBBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3geneBBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 81; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ3geneBBBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "TenseurQ3geneBBBB ";
    // puis les datas
     for (int i = 0; i< 81; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ3geneBBBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 30) A.Message(3,"operator >> (istream & entree, TenseurQ3geneBBBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ3geneBBBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 81; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ3geneBBBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ3geneBBBB ";
    // puis les datas
    for (int i = 0; i< 81; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

 
  
#endif
